# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph.util import plugin, api_pricing
from ralph_pricing.models import UsageType, DailyUsage, Device, Venture


def update_usage(device, venture, usage_type, date, value):
    if not value:
        return
    usage, created = DailyUsage.objects.get_or_create(
        date=date,
        type=usage_type,
        pricing_device=device,
    )
    usage.pricing_venture = venture
    usage.value = value
    usage.save()


def update(data, usages, date):
    if data.get('device_id') is not None:
        device, created = Device.objects.get_or_create(
            device_id=data['device_id'],
        )
    else:
        return
    if data.get('venture_id') is not None:
        venture, created = Venture.objects.get_or_create(
            venture_id=data['venture_id'],
        )
    else:
        venture = None
    for key, usage in usages.iteritems():
        update_usage(device, venture, usage, date, data.get(key))

def get_usages():
    cpu_usage, created = UsageType.objects.get_or_create(
        name="Virtual CPU cores",
    )
    cpu_usage.average = True
    cpu_usage.save()
    memory_usage, created = UsageType.objects.get_or_create(
        name="Virtual memory MB",
    )
    memory_usage.average = True
    memory_usage.save()
    disk_usage, created = UsageType.objects.get_or_create(
        name="Virtual disk MB",
    )
    disk_usage.average = True
    disk_usage.save()
    usages = {
        'virtual_cores': cpu_usage,
        'virtual_memory': memory_usage,
        'virtual_disk': disk_usage,
    }
    return usages


@plugin.register(chain='pricing', requires=['devices'])
def virtual_usages(**kwargs):
    """Updates the virtual usages from Ralph."""

    date = kwargs['today']
    usages = get_usages()
    for data in api_pricing.get_virtual_usages():
        update(data, usages, date)
    return True, 'virtual usages updated', kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph.util import plugin
from ralph_assets.api_pricing import get_asset_parts
from ralph_pricing.models import Device, DailyPart


def update_assets_parts(data, date):
    if data['asset_id'] is None and date['ralph_id'] is None:
        return False
    device, created_device = Device.objects.get_or_create(
        device_id=data['ralph_id'],
    )
    daily, created = DailyPart.objects.get_or_create(
        date=date,
        asset_id=data['asset_id'],
        defaults={
            'pricing_device_id': device.id,
        },
    )
    daily.price = data['price']
    daily.deprecation_rate = data['deprecation_rate']
    daily.name = data['model']
    daily.is_deprecated = data['is_deprecated']
    daily.save()
    return created_device


@plugin.register(chain='pricing', requires=['devices'])
def parts(**kwargs):
    """Updates the devices from Ralph Assets."""
    date = kwargs['today']
    count = sum(update_assets_parts(data, date) for data in get_asset_parts())
    return True, '%d new devices' % count, kwargs



simple_tenant_usage_data = \
[{
    u'total_volume_gb_usage': 480.0,
    u'total_memory_mb_usage': 393216.0,
    u'total_vcpus_usage': 192.0,
    u'start': u'2012-09-05 00:00:00',
    u'tenant_id': u'e638ade19b0de826ee1391ee8e84dd7e',
    u'stop': u'2012-09-06 00:00:00',
    u'total_hours': 48.0,
    u'total_local_gb_usage': 768.0,
    u'total_images_gb_usage': 199.3,
},
{
    u'total_volume_gb_usage': 28800.0,
    u'total_memory_mb_usage': 786432.0,
    u'total_vcpus_usage': 384.0,
    u'start': u'2012-09-05 00:00:00',
    u'tenant_id': u'c6b791dae6baa644a71bed1a71615f2d',
    u'stop': u'2012-09-06 00:00:00',
    u'total_hours': 96.0,
    u'total_local_gb_usage': 1536.0,
    u'total_images_gb_usage': 315.0,
},
{
    u'total_volume_gb_usage': 0,
    u'total_memory_mb_usage': 98304.0,
    u'total_vcpus_usage': 48.0,
    u'start': u'2012-09-05 00:00:00',
    u'tenant_id': u'8c9b6d8553c91ae6ad5023cfc1febb1c',
    u'stop': u'2012-09-06 00:00:00',
    u'total_hours': 48.0,
    u'total_images_gb_usage': 45.0,
    u'total_local_gb_usage': 216.0,
}]
tenants_usages_data = {u'tenant_usages': simple_tenant_usage_data}
tenants_data = \
{u'tenants': [
    {
        u'enabled': True,
        u'name': u'test_venture1',
        u'id': u'e638ade19b0de826ee1391ee8e84dd7e',
        u'description': 'venture:test_venture1;',
    },
    {
        u'enabled': True,
        u'name': u'test_venture2',
        u'id': u'c6b791dae6baa644a71bed1a71615f2d',
        u'description': 'venture:test_venture2;',
    },
    {
        u'enabled': False,
        u'name': u'test_venture3',
        u'id': u'8c9b6d8553c91ae6ad5023cfc1febb1c',
        u'description': 'fake description;'
    }
]}
tenants = {
    'e638ade19b0de826ee1391ee8e84dd7e': 'test_venture1',
    'c6b791dae6baa644a71bed1a71615f2d': 'test_venture2'
}



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import collections
import logging


from django.conf import settings

from ralph.util import plugin
from ralph_pricing.models import UsageType, Venture, DailyUsage
from ralph_pricing.openstack import OpenStack


logger = logging.getLogger(__name__)


def set_usages(venture_symbol, data, date):
    try:
        venture = Venture.objects.get(symbol=venture_symbol)
    except Venture.DoesNotExist:
        logger.error('Venture: %s does not exist' % venture_symbol)
        return

    def set_usage(name, key, venture, multiplier):
        if key not in data:
            return
        usage_type, created = UsageType.objects.get_or_create(name=name)
        usage, created = DailyUsage.objects.get_or_create(
            date=date,
            type=usage_type,
            pricing_venture=venture,
        )
        usage.value = data[key] / multiplier
        usage.save()
    if venture:
        set_usage(
            'OpenStack 10000 Memory GiB Hours',
            'total_memory_mb_usage',
            venture,
            1024,
        )
        set_usage(
            'OpenStack 10000 CPU Hours',
            'total_vcpus_usage',
            venture,
            1,
        )
        set_usage(
            'OpenStack 10000 Disk GiB Hours',
            'total_local_gb_usage',
            venture,
            1,
        )
        set_usage(
            'OpenStack 10000 Volume GiB Hours',
            'total_volume_gb_usage',
            venture,
            1,
        )
        set_usage(
            'OpenStack 10000 Images GiB Hours',
            'total_images_gb_usage',
            venture,
            1,
        )


@plugin.register(chain='pricing', requires=['ventures'])
def openstack(**kwargs):
    """Updates OpenStack usage per Venture"""
    if settings.OPENSTACK_URL is None:
        return False, 'Not configured.', kwargs
    tenants = collections.defaultdict(lambda: collections.defaultdict(dict))
    date = kwargs['today']
    end = date
    start = end - datetime.timedelta(days=1)
    ventures = {}
    for region in getattr(settings, 'OPENSTACK_REGIONS', ['']):
        stack = OpenStack(
            settings.OPENSTACK_URL,
            settings.OPENSTACK_USER,
            settings.OPENSTACK_PASS,
            region=region,
        )
        ventures.update(stack.get_ventures())
        for data in stack.simple_tenant_usage(start, end):
            tenants[data['tenant_id']][region].update(data)
    for url, query in getattr(settings, 'OPENSTACK_EXTRA_QUERIES', []):
        for data in stack.query(
            query,
            url=url,
            start=start.strftime('%Y-%m-%dT%H:%M:%S'),
            end=end.strftime('%Y-%m-%dT%H:%M:%S'),
        ):
            tenants[data['tenant_id']][url].update(data)
    for tenant_id, regions in tenants.iteritems():
        for region, data in regions.iteritems():
            venture_symbol = ventures.get(data['tenant_id'])
            if venture_symbol:
                set_usages(venture_symbol, data, date)
    return True, 'Openstack usages were saved', kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.contrib import admin
from lck.django.common.admin import ModelAdmin

from ralph_pricing import models


def register(model):
    def decorator(cls):
        admin.site.register(model, cls)
        return cls
    return decorator


class DailyDeviceInline(admin.TabularInline):
    model = models.DailyDevice


class DailyPartInline(admin.TabularInline):
    model = models.DailyPart


class DailyUsageInline(admin.TabularInline):
    model = models.DailyUsage


@register(models.Device)
class DeviceAdmin(ModelAdmin):
    inlines = [DailyDeviceInline, DailyPartInline, DailyUsageInline]


class ExtraCostInline(admin.TabularInline):
    model = models.ExtraCost


@register(models.Venture)
class VentureAdmin(ModelAdmin):
    inlines = [ExtraCostInline]


class UsagePriceInline(admin.TabularInline):
    model = models.UsagePrice


@register(models.UsageType)
class UsageTypeAdmin(ModelAdmin):
    inlines = [UsagePriceInline]


@register(models.ExtraCostType)
class ExtraCostTypeAdmin(ModelAdmin):
    inlines = [ExtraCostInline]


@register(models.SplunkName)
class SplunkNameAdmin(ModelAdmin):
    list_display = ('splunk_name', 'pricing_device')



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph.util import plugin, api_pricing
from ralph_pricing.models import Venture


def update_venture(data):
    venture, created = Venture.objects.get_or_create(
        venture_id=data['id'],
    )
    venture.name = data['name']
    venture.department = data['department']
    venture.symbol = data['symbol']
    venture.business_segment = data['business_segment']
    venture.profit_center = data['profit_center']
    if data.get('parent_id'):
        parent, parent_created = Venture.objects.get_or_create(
            venture_id=data['parent_id'],
        )
        venture.parent = parent
    else:
        parent_created = False
    venture.save()
    return created + parent_created


@plugin.register(chain='pricing', requires=[])
def ventures(**kwargs):
    """Updates the ventures from Ralph."""

    count = sum(update_venture(data) for data in api_pricing.get_ventures())
    return True, '%d new ventures' % count, kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.db.transaction import commit_on_success

from ralph.util import plugin
from ralph_assets.api_pricing import get_assets
from ralph_pricing.models import Device, DailyDevice

@commit_on_success
def update_assets(data, date):
    created = False
    if not data['ralph_id']:
        return False
    try:
        old_device = Device.objects.exclude(
            device_id=data['ralph_id'],
        ).get(
            asset_id=data['asset_id'],
        )
    except Device.DoesNotExist:
        pass
    else:
        old_device.asset_id = None
        old_device.save()
    try:
        device = Device.objects.get(device_id=data['ralph_id'])
    except Device.DoesNotExist:
        created = True
        device = Device()
        device.device_id = data['ralph_id']
    device.asset_id = data['asset_id']
    device.slots = data['slots']
    device.sn = data['sn']
    device.barcode = data['barcode']
    device.save()
    daily, daily_created = DailyDevice.objects.get_or_create(
        date=date,
        pricing_device=device,
    )
    daily.price = data['price']
    daily.deprecation_rate = data['deprecation_rate']
    daily.is_deprecated = data['is_deprecated']
    daily.save()
    return created


@plugin.register(chain='pricing', requires=['devices'])
def assets(**kwargs):
    """Updates the devices from Ralph Assets."""

    date = kwargs['today']
    count = sum(update_assets(data, date) for data in get_assets())
    return True, '%d new devices' % count, kwargs



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import mock


from django.conf import settings
from django.test import TestCase

from ralph_pricing.models import DailyDevice, Device, DailyUsage, Venture
from ralph_pricing.plugins.splunk import (
    splunk as splunk_runner,
)
from ralph_pricing.tests.samples.splunk import hosts_usages_data


class MockSplunk(object):
    """ Simple mock for Splunk API library """
    def __init__(self, *args, **kwargs):
        pass

    @property
    def progress(self, *args, **kwargs):
        return 100.1

    @property
    def results(self, *args, **kwargs):
        return hosts_usages_data

    def start(self, *args, **kwargs):
        pass


class TestSplunkPluginTest(TestCase):
    """ Splunk costs Test Case """
    def setUp(self):
        self.splunk_venture = Venture(
            name='Splunk unknown usage',
            venture_id=666,
            symbol='splunk_unknown_usage',
        )
        self.splunk_venture.save()
        venture1 = Venture(name='venture1', venture_id=111, symbol='venture1')
        venture1.save()
        venture2 = Venture(name='venture2', venture_id=222, symbol='venture2')
        venture2.save()

        self.device1 = Device(name='test_host1', device_id=1)
        self.device1.save()
        self.device2 = Device(name='test_host2', device_id=2)
        self.device2.save()

        daily_device1 = DailyDevice(
            date=datetime.datetime.today(),
            name='test_host1',
            pricing_venture=venture1,
            pricing_device=self.device1,
        )
        daily_device1.save()
        daily_device2 = DailyDevice(
            date=datetime.datetime.today(),
            name='test_host2',
            pricing_venture=venture2,
            pricing_device=self.device2,
        )
        daily_device2.save()

    def test_set_usages(self):
        """ OpenStack usages Test Case """
        # fake setting need to run plugin
        settings.SPLUNK_HOST = 'test'
        settings.SPLUNK_USER = 'test'
        settings.SPLUNK_PASSWORD = 'test'
        with mock.patch('ralph_pricing.plugins.splunk.Splunk') as Splunk:
            Splunk.side_effect = MockSplunk
            splunk_runner(today=datetime.date.today())
            usage_device1 = DailyUsage.objects.get(pricing_device=self.device1)
            usage_device2 = DailyUsage.objects.get(pricing_device=self.device2)
            usage_splunk_venture = DailyUsage.objects.get(
                pricing_venture=self.splunk_venture,
            )
            self.assertEqual(usage_device1.value, 10318.234132)
            self.assertEqual(usage_device2.value, 1326.640829)
            self.assertEqual(usage_splunk_venture.value, 1048.363416)

    def test_fail_plugin(self):
            """ Testing not configured plugin """
            with mock.patch('ralph_pricing.plugins.splunk.Splunk') as Splunk:
                Splunk.side_effect = MockSplunk
                status, message, arg = splunk_runner(
                    today=datetime.datetime.today(),
                )
                self.assertFalse(status)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from decimal import Decimal as D
from dateutil import rrule

from django.db import models as db
from django.utils.translation import ugettext_lazy as _

from mptt.models import MPTTModel, TreeForeignKey


PRICE_DIGITS = 16
PRICE_PLACES = 6


def get_usages_count_price(query, start, end):
    days = (end - start).days + 1
    count = 0
    price = D(0)
    for day in rrule.rrule(rrule.DAILY, dtstart=start, until=end):
        daily_count = 0
        for usage in query.filter(date=day):
            daily_count = usage.value
            if usage.type.average:
                count += daily_count / days
            else:
                count += daily_count
            try:
                daily_price = usage.type.get_price_at(day)
            except (
                UsagePrice.DoesNotExist,
                UsagePrice.MultipleObjectsReturned,
            ):
                price = None
            else:
                if price is not None:
                    price += D(daily_count) * daily_price
    return count, price


class Device(db.Model):
    name = db.CharField(verbose_name=_("name"), max_length=255)
    sn = db.CharField(max_length=200, null=True, blank=True)
    barcode = db.CharField(max_length=200, null=True, blank=True, default=None)
    device_id = db.IntegerField(
        verbose_name=_("device id"),
        unique=True,
    )
    asset_id = db.IntegerField(
        verbose_name=_("asset id"),
        unique=True,
        null=True,
        blank=True,
        default=None,
    )
    is_virtual = db.BooleanField(verbose_name=_("is virtual"), default=False)
    is_blade = db.BooleanField(verbose_name=_("is blade"), default=False)
    slots = db.FloatField(verbose_name=_("slots"), default=0)

    class Meta:
        verbose_name = _("device")
        verbose_name_plural = _("devices")

    def __unicode__(self):
        return '{} - {}'.format(self.name, self.device_id)

    def get_deprecated_status(self, start, end, venture):
        query = self.dailydevice_set.filter(
            pricing_venture=venture,
            date__gte=start,
            date__lte=end,
        )
        statuses = []
        last = None
        for daily_device in query:
            status = daily_device.is_deprecated
            if status != last:
                data = '%s: %s' % (daily_device.date, status)
                statuses.append(data)
            last = status
        return " ".join(statuses)

    def get_daily_usages(self, start, end):
        query = DailyUsage.objects.filter(pricing_device=self)
        for type_ in UsageType.objects.all():
            count, price = get_usages_count_price(
                query.filter(type=type_),
                start,
                end,
            )
            if count or price:
                yield {
                    'name': type_.name,
                    'count': count,
                    'price': price,
                }

    def get_daily_parts(self, start, end):
        ids = self.dailypart_set.filter(
            pricing_device=self.id,
            date__gte=start,
            date__lte=end,
        ).values_list('asset_id', flat=True)
        parts = []
        for asset_id in set(ids):
            price, cost = get_daily_price_cost(asset_id, self, start, end)
            parts.append(
                {
                    'name': self.dailypart_set.filter(
                        asset_id=asset_id
                    )[0].name,
                    'price': price,
                    'cost': cost,
                }
            )
        return parts


class ParentDevice(Device):
    class Meta:
        proxy = True


class Venture(MPTTModel):
    venture_id = db.IntegerField()
    name = db.CharField(
        verbose_name=_("name"),
        max_length=255,
        default='',
    )
    department = db.CharField(
        verbose_name=_("department name"),
        max_length=255,
        default='',
    )
    parent = TreeForeignKey(
        'self',
        null=True,
        blank=True,
        default=None,
        related_name=_('children'),
    )
    symbol = db.CharField(
        verbose_name=_("symbol"),
        max_length=32,
        blank=True,
        default="",
    )
    business_segment = db.TextField(
        verbose_name=_("Business segment"),
        max_length=75,
        blank=True,
        default="",
    )
    profit_center = db.CharField(
        verbose_name=_("Profit center"),
        max_length=75,
        blank=True,
        default="",
    )

    class Meta:
        verbose_name = _("venture")
        verbose_name_plural = _("ventures")

    class MPTTMeta:
        order_insertion_by = ['name']

    def __unicode__(self):
        return self.name

    def _by_venture(self, query, descendants):
        if descendants:
            ventures = self.get_descendants(include_self=True)
            return query.filter(pricing_venture__in=ventures)
        return query.filter(pricing_venture=self)

    def get_assets_count_price_cost(
        self,
        start,
        end,
        descendants=False,
        zero_deprecated=True,
        device_id=False,
    ):
        days = (end - start).days + 1
        query = DailyDevice.objects.filter(pricing_device__is_virtual=False)
        if device_id:
            query = query.filter(pricing_device_id=device_id)
        else:
            query = self._by_venture(query, descendants)
        query = query.filter(date__gte=start, date__lte=end)
        query = query.select_related('pricing_device', 'parent')
        total_count = 0
        total_price = D('0')
        total_cost = D('0')
        for daily_device in query:
            asset_price, asset_cost = daily_device.get_price_cost(
                zero_deprecated,
            )
            system_price, system_cost = daily_device.get_bladesystem_price_cost(  # noqa
                zero_deprecated,
            )
            blades_price, blades_cost = daily_device.get_blades_price_cost(
                zero_deprecated,
            )
            total_price += asset_price + system_price - blades_price
            total_cost += asset_cost + system_cost - blades_cost
            total_count += 1
        return total_count / days, total_price / days, total_cost

    def get_usages_count_price(
        self, start, end, type_, descendants=False, query=None
    ):
        if query is None:
            query = DailyUsage.objects
        query = query.filter(type=type_)
        query = self._by_venture(query, descendants)
        query = query.filter(date__gte=start, date__lte=end)
        return get_usages_count_price(query, start, end)

    def get_extra_costs(self, start, end, type_, descendants=False):
        price = D('0')
        query = ExtraCost.objects.filter(type=type_)
        query = self._by_venture(query, descendants)
        for day in rrule.rrule(rrule.DAILY, dtstart=start, until=end):
            extras = query.filter(start__lte=day, end__gte=day)
            price += extras.aggregate(db.Sum('price'))['price__sum'] or 0
        return price

    def get_extracost_details(self, start, end):
        extracost = ExtraCost.objects.filter(
            start__gte=start,
            end__lte=end,
            pricing_venture=self,
        )
        return extracost

    def get_daily_usages(self, start, end):
        query = DailyUsage.objects.filter(pricing_device=None)
        query = self._by_venture(query, descendants=False)
        for type_ in UsageType.objects.all():
            count, price = get_usages_count_price(
                query.filter(type=type_),
                start,
                end,
            )
            if count or price:
                yield {
                    'name': type_.name,
                    'count': count,
                    'price': price,
                }


class DailyPart(db.Model):
    date = db.DateField()
    name = db.CharField(verbose_name=_("name"), max_length=255)
    pricing_device = db.ForeignKey(Device)
    asset_id = db.IntegerField()
    price = db.DecimalField(
        max_digits=PRICE_DIGITS,
        decimal_places=PRICE_PLACES,
        verbose_name=_("price"),
        default=0,
    )
    deprecation_rate = db.DecimalField(
        max_digits=PRICE_DIGITS,
        decimal_places=PRICE_PLACES,
        verbose_name=_("deprecation rate"),
        default=0,
    )
    is_deprecated = db.BooleanField(
        verbose_name=_("is deprecated"),
        default=False,
    )

    class Meta:
        verbose_name = _("daily part")
        verbose_name_plural = _("daily parts")
        unique_together = ('date', 'asset_id')
        ordering = ('asset_id', 'pricing_device', 'date')

    def __unicode__(self):
        return '{} ({})'.format(self.name, self.date)

    def get_price_cost(self, zero_deprecated=True):
        if zero_deprecated and self.is_deprecated:
            return D('0'), D('0')
        total_cost = self.price * self.deprecation_rate / 36500
        return self.price, total_cost


def get_daily_price_cost(asset_id, device, start, end):
    days = (end - start).days + 1
    query = DailyPart.objects.filter(
        asset_id=asset_id,
        pricing_device=device,
        date__gte=start,
        date__lte=end,
    )
    total_price, total_cost = 0, 0
    for daily in query:
        price, cost = daily.get_price_cost()
        total_price += price
        total_cost += cost
    return total_price / days, total_cost


class DailyDevice(db.Model):
    date = db.DateField()
    name = db.CharField(verbose_name=_("name"), max_length=255)
    pricing_device = db.ForeignKey(
        Device,
        verbose_name=_("pricing device"),
    )
    parent = db.ForeignKey(
        # Note: this is only relevant for blade and virtual servers.
        ParentDevice,
        verbose_name=_("parent"),
        related_name='child_set',
        null=True,
        blank=True,
        default=None,
        on_delete=db.SET_NULL,
    )
    price = db.DecimalField(
        max_digits=PRICE_DIGITS,
        decimal_places=PRICE_PLACES,
        verbose_name=_("price"),
        default=0,
    )
    deprecation_rate = db.DecimalField(
        max_digits=PRICE_DIGITS,
        decimal_places=PRICE_PLACES,
        verbose_name=_("deprecation rate"),
        default=0,
    )
    pricing_venture = db.ForeignKey(
        Venture,
        verbose_name=_("venture"),
        null=True,
        blank=True,
        default=None,
        on_delete=db.SET_NULL,
    )
    is_deprecated = db.BooleanField(
        verbose_name=_("is deprecated"),
        default=False,
    )

    class Meta:
        verbose_name = _("daily device")
        verbose_name_plural = _("daily devices")
        unique_together = ('date', 'pricing_device')

    def __unicode__(self):
        return '{} ({})'.format(self.name, self.date)

    def get_price_cost(self, zero_deprecated=True):
        """
        Return the price and daily cost of the device on that day.
        This only includes the price of the asset itself, not the
        prices of its parents (in case of blade systems).
        """

        total_price = D('0')
        total_cost = D('0')
        # If the device has parts, sum them up
        for daily_part in self.pricing_device.dailypart_set.filter(
                date=self.date,
        ):
            price, cost = daily_part.get_price_cost()
            total_price += price
            total_cost += cost
        # Otherwise just take the price and cost of the device
        if zero_deprecated and self.is_deprecated:
            return D('0'), D('0')
        if not total_price and not total_cost:
            total_price = self.price
            total_cost = self.price * self.deprecation_rate / 36500
        return total_price, total_cost

    def get_bladesystem_price_cost(
        self,
        zero_deprecated=True,
        daily_parent=None,
    ):
        """
        Return the fraction of the price and cost of the blade system
        containing this device.
        """

        total_price = D('0')
        total_cost = D('0')
        if (
            not self.pricing_device.is_blade or
            not self.parent or
            not self.pricing_device.slots
        ):
            return total_price, total_cost
        if not daily_parent:
            try:
                daily_parent = self.parent.dailydevice_set.filter(
                    date=self.date,
                )
            except DailyDevice.DoesNotExist:
                pass
        if daily_parent and daily_parent.pricing_device.slots:
            system_price, system_cost = daily_parent.get_price_cost(
                zero_deprecated,
            )
            system_fraction = (
                D(self.pricing_device.slots) /
                D(daily_parent.pricing_device.slots)
            )
            total_cost += system_cost * system_fraction
            total_price += system_price * system_fraction
        return total_price, total_cost

    def get_blades_price_cost(self, zero_deprecated=True):
        """
        Return the prices and costs that the blades contained in a blade
        system subtract from that blade system.
        """

        total_price = D('0')
        total_cost = D('0')
        if self.pricing_device.slots and not self.pricing_device.is_blade:
            for blade in self.pricing_device.children_set.filter(
                    date=self.date,
                    pricing_device__is_blade=True,
            ):
                price, cost = blade.get_bladesystem_price_cost(
                    zero_deprecated,
                    self,
                )
                total_price += price
                total_cost += cost
        return total_price, total_cost


class UsageType(db.Model):
    name = db.CharField(verbose_name=_("name"), max_length=255, unique=True)
    average = db.BooleanField(
        verbose_name=_("Average the values over multiple days"),
        default=False,
    )
    show_value_percentage = db.BooleanField(
        verbose_name=_("Show percentage of value"),
        default=False,
    )
    show_price_percentage = db.BooleanField(
        verbose_name=_("Show percentage of price"),
        default=False,
    )

    class Meta:
        verbose_name = _("usage type")
        verbose_name_plural = _("usage types")

    def __unicode__(self):
        return self.name

    def get_price_at(self, date):
        return self.usageprice_set.get(start__lte=date, end__gte=date).price


class UsagePrice(db.Model):
    type = db.ForeignKey(UsageType, verbose_name=_("type"))
    price = db.DecimalField(
        max_digits=PRICE_DIGITS,
        decimal_places=PRICE_PLACES,
        verbose_name=_("price"),
    )
    start = db.DateField()
    end = db.DateField()

    class Meta:
        verbose_name = _("usage price")
        verbose_name_plural = _("usage prices")
        unique_together = [
            ('start', 'type'),
            ('end', 'type'),
        ]
        ordering = ('type', 'start')

    def __unicode__(self):
        return '{} ({}-{})'.format(self.type, self.start, self.end)


class DailyUsage(db.Model):
    date = db.DateField()
    pricing_venture = db.ForeignKey(
        Venture,
        verbose_name=_("venture"),
        null=True,
        blank=True,
        default=None,
        on_delete=db.SET_NULL,
    )
    pricing_device = db.ForeignKey(
        Device,
        verbose_name=_("pricing device"),
        null=True,
        blank=True,
        default=None,
        on_delete=db.SET_NULL,
    )
    value = db.FloatField(verbose_name=_("value"), default=0)
    type = db.ForeignKey(UsageType, verbose_name=_("type"))

    class Meta:
        verbose_name = _("daily usage")
        verbose_name_plural = _("daily usages")
        unique_together = ('date', 'pricing_device', 'type')
        ordering = ('pricing_device', 'type', 'date')

    def __unicode__(self):
        return '{0}/{1} ({2}) {3}'.format(
            self.pricing_device,
            self.type,
            self.date,
            self.value,
        )


class ExtraCostType(db.Model):
    name = db.CharField(verbose_name=_("name"), max_length=255, unique=True)

    class Meta:
        verbose_name = _("extra cost type")
        verbose_name_plural = _("extra cost types")

    def __unicode__(self):
        return self.name

    def get_cost_at(self, date):
        return 1  # XXX


class ExtraCost(db.Model):
    start = db.DateField()
    end = db.DateField()
    type = db.ForeignKey(ExtraCostType, verbose_name=_("type"))
    price = db.DecimalField(
        max_digits=PRICE_DIGITS,
        decimal_places=PRICE_PLACES,
        verbose_name=_("price"),
    )
    pricing_venture = db.ForeignKey(Venture, verbose_name=_("venture"))

    class Meta:
        verbose_name = _("extra cost")
        verbose_name_plural = _("extra costs")
        unique_together = [
            ('start', 'pricing_venture', 'type'),
            ('end', 'pricing_venture', 'type'),
        ]

    def __unicode__(self):
        return '{}/{} ({} - {})'.format(
            self.pricing_venture,
            self.type,
            self.start,
            self.end,
        )


class SplunkName(db.Model):
    splunk_name = db.CharField(
        verbose_name=_("Splunk name"),
        max_length=255,
        blank=False,
        unique=True,
    )
    pricing_device = db.ForeignKey(
        Device,
        verbose_name=_("pricing device"),
        null=True,
        blank=True,
        default=None,
        on_delete=db.SET_NULL,
    )

    class Meta:
        unique_together = ("splunk_name", "pricing_device")



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime

from django.utils.translation import ugettext_lazy as _

from ralph_pricing.views.reports import Report, currency
from ralph_pricing.models import UsageType, ExtraCostType, Venture
from ralph.business.models import Venture as ralph_venture
from ralph_pricing.forms import DateRangeForm


class AllVentures(Report):
    template_name = 'ralph_pricing/ventures_all.html'
    Form = DateRangeForm
    section = 'all-ventures'
    report_name = _('All Ventures Report')

    @staticmethod
    def get_data(start, end, show_in_ralph=False, **kwargs):
        ventures = Venture.objects.order_by('name')
        total_count = ventures.count() + 1  # additional step for post-process
        data = []
        totals = {}
        values = []
        for i, venture in enumerate(ventures):
            show_venture = ralph_venture.objects.get(
                id=venture.venture_id
            ).show_in_ralph
            if show_in_ralph and not show_venture:
                continue
            values_row = {}
            values.append(values_row)
            count, price, cost = venture.get_assets_count_price_cost(
                start, end,
            )
            path = '/'.join(
                v.name for v in venture.get_ancestors(include_self=True),
            )
            row = [
                venture.venture_id,
                path,
                show_in_ralph,
                venture.department,
                venture.business_segment,
                venture.profit_center,
                count,
                currency(price),
                currency(cost),
            ]
            column = len(row)
            for usage_type in UsageType.objects.order_by('name'):
                count, price = venture.get_usages_count_price(
                    start,
                    end,
                    usage_type,
                )
                row.append(count)
                column += 1
                if usage_type.show_value_percentage:
                    row.append('')
                    totals[column] = totals.get(column, 0) + count
                    values_row[column] = count
                    column += 1
                if price is None:
                    row.append('NO PRICE')
                else:
                    row.append(currency(price))
                column += 1
                if usage_type.show_price_percentage:
                    row.append('')
                    totals[column] = totals.get(column, 0) + count
                    values_row[column] = price
                    column += 1
            for extra_cost_type in ExtraCostType.objects.order_by('name'):
                row.append(currency(venture.get_extra_costs(
                    start,
                    end,
                    extra_cost_type,
                )))
            progress = (100 * i) // total_count
            data.append(row)
            yield min(progress, 99), data
        for row, values_row in zip(data, values):
            for column, total in totals.iteritems():
                if total:
                    row[column] = '{:.2f}%'.format(
                        100 * values_row[column] / total,
                    )
        yield 100, data

    @staticmethod
    def get_header(**kwargs):
        header = [
            _("ID"),
            _("Venture"),
            _("Active at %s" % datetime.date.today()),
            _("Department"),
            _("Business segment"),
            _("Profit center"),
            _("Assets count"),
            _("Assets price"),
            _("Assets cost"),
        ]
        for usage_type in UsageType.objects.order_by('name'):
            header.append(_("{} count").format(usage_type.name))
            if usage_type.show_value_percentage:
                header.append(_("{} count %").format(usage_type.name))
            header.append(_("{} price").format(usage_type.name))
            if usage_type.show_price_percentage:
                header.append(_("{} price %").format(usage_type.name))
        for extra_cost_type in ExtraCostType.objects.order_by('name'):
            header.append(extra_cost_type.name)
        return header


class TopVentures(AllVentures):
    template_name = 'ralph_pricing/ventures_top.html'
    section = 'top-ventures'
    report_name = _('Top Ventures Report')

    @staticmethod
    def get_data(start, end, show_in_ralph=False, **kwargs):
        ventures = Venture.objects.root_nodes().order_by('name')
        total_count = ventures.count() + 1  # additional step for post-process
        data = []
        totals = {}
        values = []
        for i, venture in enumerate(ventures):
            show_venture = ralph_venture.objects.get(
                id=venture.venture_id
            ).show_in_ralph
            if show_in_ralph and not show_venture:
                continue
            values_row = {}
            values.append(values_row)
            count, price, cost = venture.get_assets_count_price_cost(
                start,
                end,
                descendants=True,
            )
            row = [
                venture.venture_id,
                venture.name,
                ralph_venture.objects.get(id=venture.venture_id).show_in_ralph,
                venture.department,
                venture.business_segment,
                venture.profit_center,
                count,
                currency(price),
                currency(cost),
            ]
            column = len(row)
            for usage_type in UsageType.objects.order_by('name'):
                count, price = venture.get_usages_count_price(
                    start,
                    end,
                    usage_type,
                    descendants=True,
                )
                row.append(count)
                column += 1
                if usage_type.show_value_percentage:
                    row.append('')
                    totals[column] = totals.get(column, 0) + count
                    values_row[column] = count
                    column += 1
                row.append(currency(price))
                column += 1
                if usage_type.show_price_percentage:
                    row.append('')
                    totals[column] = totals.get(column, 0) + price
                    values_row[column] = price
                    column += 1
            for extra_cost_type in ExtraCostType.objects.order_by('name'):
                row.append(currency(venture.get_extra_costs(
                    start,
                    end,
                    extra_cost_type,
                    descendants=True,
                )))
            progress = (100 * i) // total_count
            data.append(row)
            yield min(progress, 99), data
        for row, values_row in zip(data, values):
            for column, total in totals.iteritems():
                if total:
                    row[column] = '{:.2f}%'.format(
                        100 * values_row[column] / total,
                    )
        yield 100, data



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph_pricing.models import Venture, UsageType

from bob.menu import MenuItem


def ventures_menu(href='', selected=None):
    top_items = []
    stack = [
        (None, top_items, Venture.objects.root_nodes()),
    ]
    while stack:
        parent, items, ventures = stack.pop()
        for venture in ventures.order_by('name'):
            item = MenuItem(
                venture.name,
                name='{}'.format(venture.id),
                subitems=[],
                fugue_icon='fugue-store-medium',
                indent=' ',
                href='{}/{}/'.format(href, venture.id),
                collapsed=True,
                collapsible=True,
            )
            item.parent = parent
            items.append(item)
            stack.append((item, item.subitems, venture.get_children()))
            if item.name == selected:
                while item:
                    item.kwargs['collapsed'] = False
                    item = item.parent
    return top_items


def usages_menu(href='', selected=None):
    items = [
        MenuItem(
            usage_type.name,
            name=usage_type.name,
            subitems=[],
            fugue_icon='fugue-beaker',
            href='{}/{}/'.format(href, usage_type.name),
        ) for usage_type in UsageType.objects.order_by('name')
    ]
    return items



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import decimal

from django.test import TestCase

from ralph_pricing import models


class TestModels(TestCase):
    def test_device_model(self):
        device = models.Device(device_id=3)
        device.save()
        device = models.Device.objects.get(device_id=3)
        self.assertEquals(device.device_id, 3)

    def test_venture_model(self):
        venture = models.Venture(venture_id=3)
        venture.save()
        venture = models.Venture.objects.get(venture_id=3)
        self.assertEquals(venture.venture_id, 3)

    def test_dailypart_model(self):
        device = models.Device(device_id=2)
        device.save()
        part = models.DailyPart(
            asset_id=4,
            pricing_device=device,
            date=datetime.date(2013, 4, 25),
            name='ziew',
            price=5,
        )
        part.save()
        self.assertEquals(part.date, datetime.date(2013, 4, 25))
        self.assertEquals(part.pricing_device.device_id, 2)
        self.assertEquals(part.asset_id, 4)
        self.assertEquals(part.price, 5)

    def test_dailydevice_model(self):
        device = models.Device(device_id=2)
        device.save()
        daily = models.DailyDevice(
            pricing_device=device,
            date=datetime.date(2013, 4, 25),
            name='ziew',
        )
        daily.save()
        self.assertEquals(daily.date, datetime.date(2013, 4, 25))
        self.assertEquals(daily.pricing_device.device_id, 2)

    def test_usage_models(self):
        type_ = models.UsageType(name='ziew')
        type_.save()
        price = models.UsagePrice(
            type=type_,
            price=3,
            start=datetime.date(2013, 4, 24),
            end=datetime.date(2013, 4, 26),
        )
        price.save()
        usage = models.DailyUsage(
            date=datetime.date(2013, 4, 25),
            value=3,
            type=type_,
        )
        usage.save()
        self.assertEquals(usage.value, 3)
        self.assertEquals(usage.type, type_)
        self.assertEquals(price.price, 3)
        self.assertEquals(type_.name, 'ziew')

    def test_extra_cost_models(self):
        type_ = models.ExtraCostType(name='ziew')
        type_.save()
        venture = models.Venture(venture_id=3)
        venture.save()
        cost = models.ExtraCost(
            type=type_,
            start=datetime.datetime(2013, 4, 24),
            end=datetime.datetime(2013, 4, 26),
            price=3,
            pricing_venture=venture,
        )
        cost.save()
        self.assertEquals(type_.name, 'ziew')
        self.assertEquals(venture.venture_id, 3)
        self.assertEquals(cost.type, type_)
        self.assertEquals(cost.start, datetime.datetime(2013, 4, 24))
        self.assertEquals(cost.end, datetime.datetime(2013, 4, 26))
        self.assertEquals(cost.price, 3)
        self.assertEquals(cost.pricing_venture, venture)


class TestPrices(TestCase):
    def test_asset_count_price(self):
        day = datetime.date(2013, 4, 25)
        venture = models.Venture(venture_id=3)
        venture.save()
        device = models.Device(
            device_id=3,
            asset_id=5,
        )
        device.save()
        daily = models.DailyDevice(
            pricing_device=device,
            date=day,
            name='ziew',
            price='1337',
            pricing_venture=venture,
        )
        daily.save()
        subventure = models.Venture(venture_id=2, parent=venture)
        subventure.save()
        other_device = models.Device(
            device_id=2,
            asset_id=3,
        )
        other_device.save()
        other_daily = models.DailyDevice(
            pricing_device=other_device,
            date=day,
            name='ziew',
            price='833833',
            pricing_venture=subventure,
        )
        other_daily.save()
        count, price, cost = venture.get_assets_count_price_cost(day, day)
        self.assertEquals(count, 1)
        self.assertEquals(price, decimal.Decimal('1337'))
        count, price, cost = venture.get_assets_count_price_cost(
            day, day, True,
        )
        self.assertEquals(count, 2)
        self.assertEquals(price, decimal.Decimal('835170'))

    def test_usages_count_price(self):
        day = datetime.date(2013, 4, 25)
        venture = models.Venture(venture_id=3)
        venture.save()
        usage_type = models.UsageType(name='waciki')
        usage_type.save()
        daily_usage = models.DailyUsage(
            type=usage_type,
            value=32,
            date=day,
            pricing_venture=venture,
        )
        daily_usage.save()
        other_day = datetime.date(2013, 4, 24)
        daily_usage = models.DailyUsage(
            type=usage_type,
            value=32,
            date=other_day,
            pricing_venture=venture,
        )
        daily_usage.save()
        usage_price = models.UsagePrice(
            start=day,
            end=day,
            price=4,
            type=usage_type,
        )
        usage_price.save()
        count, price = venture.get_usages_count_price(day, day, usage_type)
        self.assertEquals(count, 32)
        self.assertEquals(price, decimal.Decimal('128'))
        day = datetime.date(2013, 4, 26)
        count, price = venture.get_usages_count_price(day, day, usage_type)
        self.assertEquals(count, 0)
        self.assertEquals(price, decimal.Decimal('0'))
        day = datetime.date(2013, 4, 24)
        count, price = venture.get_usages_count_price(day, day, usage_type)
        self.assertEquals(count, 32)
        self.assertIsNone(price)

    def test_extra_costs(self):
        day = datetime.date(2013, 4, 25)
        venture = models.Venture(venture_id=3)
        venture.save()
        extra_cost_type = models.ExtraCostType(name='waciki')
        extra_cost_type.save()
        extra_cost = models.ExtraCost(
            pricing_venture=venture,
            start=day,
            end=day,
            type=extra_cost_type,
            price='65535',
        )
        extra_cost.save()
        price = venture.get_extra_costs(day, day, extra_cost_type)
        self.assertEqual(price, decimal.Decimal('65535'))
        price = venture.get_extra_costs(
            datetime.date(2013, 4, 24),
            datetime.date(2013, 4, 24),
            extra_cost_type,
        )
        self.assertEqual(price, decimal.Decimal('0'))



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime

from django.test import TestCase

from ralph_pricing.models import DailyPart
from ralph_pricing.plugins.assets_part import update_assets_parts


class TestAssetPlugin(TestCase):
    def setUp(self):
        self.today = datetime.date.today()

    def get_asset_part(self):
        """Simulated api result"""
        yield {
            'asset_id': 1123,
            'ralph_id': 113,
            'model': 'Noname SSD',
            'price': 130,
            'is_deprecated': True,
            'deprecation_rate': 0,
        }

    def test_sync_asset_device_part(self):
        count = sum(
            update_assets_parts(
                data,
                self.today
            ) for data in self.get_asset_part()
        )
        part = DailyPart.objects.get(asset_id=1123)
        self.assertEqual(count, 1)
        self.assertEqual(part.is_deprecated, True)
        self.assertEqual(part.name, 'Noname SSD')
        self.assertEqual(part.price, 130)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.http import HttpResponseRedirect

from ralph_pricing.forms import UsagesFormSet
from ralph_pricing.menus import usages_menu
from ralph_pricing.models import UsageType
from ralph_pricing.views.base import Base


class Usages(Base):
    template_name = 'ralph_pricing/usages.html'


    def __init__(self, *args, **kwargs):
        super(Usages, self).__init__(*args, **kwargs)
        self.formset = None
        self.usage_type = None
        self.usage_type_name = None

    def init_args(self):
        self.usage_type_name = self.kwargs.get('usage')
        if self.usage_type_name is not None:
            self.usage_type = get_object_or_404(
                UsageType,
                name=self.usage_type_name,
            )

    def post(self, *args, **kwargs):
        self.init_args()
        if self.usage_type:
            self.formset = UsagesFormSet(
                self.request.POST,
                queryset=self.usage_type.usageprice_set.order_by('start'),
            )
            if self.formset.is_valid():
                for form in self.formset.extra_forms:
                    if form.has_changed():
                        form.instance.type = self.usage_type
                self.formset.save()
                messages.success(self.request, "Usage prices updated.")
                return HttpResponseRedirect(self.request.path)
            else:
                messages.error(self.request, "Please fix the errors.")
        return super(Usages, self).get(*args, **kwargs)

    def get(self, *args, **kwargs):
        self.init_args()
        if self.usage_type_name:
            self.formset = UsagesFormSet(
                queryset=self.usage_type.usageprice_set.order_by('start'),
            )
        return super(Usages, self).get(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(Usages, self).get_context_data(**kwargs)
        context.update({
            'section': 'usages',
            'sidebar_items': usages_menu(
                '/pricing/usages',
                self.usage_type_name
            ),
            'sidebar_selected': self.usage_type_name,
            'formset': self.formset,
        })
        return context



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import logging
import urllib

from django.conf import settings
from restkit import Resource, ResourceNotFound

from ralph.util import plugin
from ralph_pricing.models import DailyUsage, UsageType, Venture


logger = logging.getLogger(__name__)


def get_ventures_capacities(date, url):
    resource = Resource(url)
    ret = resource.get(urllib.quote_plus(date.strftime("%Y-%m-%d")))
    json_data = json.loads(ret.body_string())
    return json_data


def update_scaleme_usage(usage_types, date, url):
    try:
        ventures_capacity = get_ventures_capacities(date, url)
    except ResourceNotFound:
        # apache not found error
        message = 'Scaleme data for %r not found' % date.strftime("%Y-%m-%d")
        logger.error(message)
    else:
        if ventures_capacity:
            counts = {'new': 0, 'updated': 0}
            for venture_symbol, venture_usages in ventures_capacity.iteritems():  # noqa
                try:
                    venture = Venture.objects.get(symbol=venture_symbol)
                except Venture.DoesNotExist:
                    logger.error('Venture %r does not exist from Scaleme data')
                else:
                    for usage_name, usage_type in usage_types.iteritems():
                        usage, created = DailyUsage.objects.get_or_create(
                            date=date,
                            type=usage_type,
                            pricing_venture=venture,
                        )
                        usage.value = venture_usages[usage_name]
                        usage.save()
                        if created:
                            counts['new'] += 1
                        else:
                            counts['updated'] += 1
            message = 'Scaleme ussages in Ventures: {} new, {} updated'.format(
                counts['new'], counts['updated']
            )
        else:
            message = 'Scaleme data for %r not found' % date.strftime(
                "%Y-%m-%d"
            )
    return message


@plugin.register(chain='pricing', requires=['ventures'])
def scaleme(**kwargs):
    """ Update Scaleme usage per Venture """
    if not settings.SCALEME_API_URL:
        return False, "Not configured.", kwargs
    url = settings.SCALEME_API_URL
    usage_type_backend, created = UsageType.objects.get_or_create(
        name='Scaleme transforming an image 10000 events',
    )
    usage_type_cache, created = UsageType.objects.get_or_create(
        name='Scaleme serving image from cache 10000 events',
    )
    usage_types = {
        'cache': usage_type_cache,
        'backend': usage_type_backend,
    }
    date = kwargs['today']
    message = update_scaleme_usage(usage_types, date, url)
    return True, message, kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime

from decimal import Decimal
from django.test import TestCase

from ralph_pricing.models import Device, Venture, DailyDevice
from ralph_pricing.views.devices import Devices


class TestDevices(TestCase):
    fixtures = ['tests.yaml']

    def setUp(self):
        self.devices = Device.objects.all()
        self.ventures = Venture.objects.all()
        self.daily_host1 = DailyDevice.objects.filter(pricing_device=1)
        self.daily_host2 = DailyDevice.objects.filter(pricing_device=2)
        self.daily_host3 = DailyDevice.objects.filter(pricing_device=3)
        self.daily_host4 = DailyDevice.objects.filter(pricing_device=4)

    def test_fixtures(self):
        self.assertEqual(len(self.devices), 4)
        self.assertEqual(len(self.ventures), 2)
        self.assertEqual(len(self.daily_host1), 4)
        self.assertEqual(len(self.daily_host2), 4)
        self.assertEqual(len(self.daily_host3), 4)
        self.assertEqual(len(self.daily_host4), 4)

    def get_data(self, start_date, end_date, venture):
        for percent, devices_list in Devices.get_data(
            start_date,
            end_date,
            venture,
        ):
            pass
        return devices_list

    def test_device(self):
        venture = self.ventures.get(name='Infra')
        start_date = datetime.date(2013, 01, 01)
        end_date = datetime.date(2013, 01, 04)
        devices_list = self.get_data(start_date, end_date, venture)
        self.assertEqual(len(devices_list), 12)
        data = [
            [
                u'ExtraCost1 (Extra Cost)',
                u'',
                u'',
                u'2013-01-01 - 2013-01-04',
                u'',
                u'100.00 PLN',
                u'',
                u'',
                u'',
                u'',
            ],
            [
                u'ExtraCost2 (Extra Cost)',
                u'',
                u'',
                u'2013-01-01 - 2013-01-04',
                u'',
                u'200.00 PLN',
                u'',
                u'',
                u'',
                u'',
            ],
            [
                u'',
                u'',
                u'DiskShare',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                100.0,
            ],
            [
                u'',
                u'',
                u'DiskShare2',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                400.0,
            ],
            [
                u'host01.dc1',
                u'',
                u'',
                u'1234-1234-1234-1234',
                u'2345-2345-2345-2345',
                u'2013-01-01: False',
                u'275.00 PLN',
                u'3.72 PLN',
                u'',
            ],
            [
                u'',
                u'hdd-0001',
                u'',
                u'',
                u'',
                u'',
                '175.00 PLN',
                u'2.37 PLN',
                u'',
            ],
            [
                u'',
                u'ram-0001',
                u'',
                u'',
                u'',
                u'',
                u'100.00 PLN',
                u'1.35 PLN',
                u'',
            ],
            [
                u'',
                u'',
                u'DiskShare',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                350.0,
            ],
            [
                u'host02.dc1',
                u'',
                u'',
                u'2345-2345-2345-2345',
                u'3456-3456-3456-3456',
                u'2013-01-01: False',
                u'175.00 PLN',
                u'0.00 PLN',
                u'',
            ],
            [
                u'',
                u'',
                u'DiskShare2',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                975.0,
            ],
            [
                u'host03.dc1',
                u'',
                u'',
                u'3456-3456-3456-3456',
                u'4567-4567-4567-4567',
                u'2013-01-01: False 2013-01-03: True 2013-01-04: False',
                u'187.50 PLN',
                u'0.00 PLN',
                u'',
            ],
            [
                u'host04.dc1',
                u'',
                u'',
                u'4567-4567-4567-4567',
                u'5678-5678-5678-5678',
                u'2013-01-01: False',
                u'50.00 PLN',
                u'0.00 PLN',
                u''
            ],
        ]

        i = 0
        for device in devices_list:
            self.assertEqual(devices_list[i], data[i])
            i += 1
        start_date = datetime.date(2013, 01, 02)
        end_date = datetime.date(2013, 01, 04)
        devices_list = self.get_data(start_date, end_date, venture)
        self.assertEqual(len(devices_list), 6)
        data = [
            [
                u'',
                u'',
                u'DiskShare',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                100.0,
            ],
            [
                u'',
                u'',
                u'DiskShare2',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                300.0,
            ],
            [
                u'host02.dc1',
                u'',
                u'',
                u'2345-2345-2345-2345',
                u'3456-3456-3456-3456',
                u'2013-01-02: False',
                u'200.00 PLN',
                u'0.00 PLN',
                u'',
            ],
            [
                u'',
                u'',
                u'DiskShare2',
                u'',
                u'',
                u'',
                u'',
                u'0.00 PLN',
                575.0,
            ],
            [
                u'host03.dc1',
                u'',
                u'',
                u'3456-3456-3456-3456',
                u'4567-4567-4567-4567',
                u'2013-01-02: False 2013-01-03: True 2013-01-04: False',
                u'166.67 PLN',
                u'0.00 PLN',
                u'',
            ],
            [
                u'host04.dc1',
                u'',
                u'',
                u'4567-4567-4567-4567',
                u'5678-5678-5678-5678',
                u'2013-01-02: False',
                u'50.00 PLN',
                u'0.00 PLN',
                u'',
            ],
        ]
        i = 0
        for device in devices_list:
            self.assertEqual(devices_list[i], data[i])
            i += 1



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting field 'Venture.pricing_center'
        db.delete_column('ralph_pricing_venture', 'pricing_center')

        # Adding field 'Venture.profit_center'
        db.add_column('ralph_pricing_venture', 'profit_center',
                      self.gf('django.db.models.fields.CharField')(default=u'', max_length=75, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Adding field 'Venture.pricing_center'
        db.add_column('ralph_pricing_venture', 'pricing_center',
                      self.gf('django.db.models.fields.CharField')(default=u'', max_length=75, blank=True),
                      keep_default=False)

        # Deleting field 'Venture.profit_center'
        db.delete_column('ralph_pricing_venture', 'profit_center')


    models = {
        'ralph_pricing.dailydevice': {
            'Meta': {'unique_together': "((u'date', u'pricing_device'),)", 'object_name': 'DailyDevice'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'deprecation_rate': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '16', 'decimal_places': '6'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deprecated': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'child_set'", 'on_delete': 'models.SET_NULL', 'default': 'None', 'to': "orm['ralph_pricing.Device']", 'blank': 'True', 'null': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '16', 'decimal_places': '6'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.Device']"}),
            'pricing_venture': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Venture']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'})
        },
        'ralph_pricing.dailypart': {
            'Meta': {'ordering': "(u'asset_id', u'pricing_device', u'date')", 'unique_together': "((u'date', u'asset_id'),)", 'object_name': 'DailyPart'},
            'asset_id': ('django.db.models.fields.IntegerField', [], {}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'deprecation_rate': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '16', 'decimal_places': '6'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deprecated': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'price': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '16', 'decimal_places': '6'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.Device']"})
        },
        'ralph_pricing.dailyusage': {
            'Meta': {'ordering': "(u'pricing_device', u'type', u'date')", 'unique_together': "((u'date', u'pricing_device', u'type'),)", 'object_name': 'DailyUsage'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Device']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'pricing_venture': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Venture']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.UsageType']"}),
            'value': ('django.db.models.fields.FloatField', [], {'default': '0'})
        },
        'ralph_pricing.device': {
            'Meta': {'object_name': 'Device'},
            'asset_id': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'barcode': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'device_id': ('django.db.models.fields.IntegerField', [], {'unique': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_blade': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_virtual': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slots': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'sn': ('django.db.models.fields.CharField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'ralph_pricing.extracost': {
            'Meta': {'unique_together': "[(u'start', u'pricing_venture', u'type'), (u'end', u'pricing_venture', u'type')]", 'object_name': 'ExtraCost'},
            'end': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '16', 'decimal_places': '6'}),
            'pricing_venture': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.Venture']"}),
            'start': ('django.db.models.fields.DateField', [], {}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.ExtraCostType']"})
        },
        'ralph_pricing.extracosttype': {
            'Meta': {'object_name': 'ExtraCostType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'ralph_pricing.splunkname': {
            'Meta': {'unique_together': "((u'splunk_name', u'pricing_device'),)", 'object_name': 'SplunkName'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Device']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'splunk_name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'ralph_pricing.usageprice': {
            'Meta': {'ordering': "(u'type', u'start')", 'unique_together': "[(u'start', u'type'), (u'end', u'type')]", 'object_name': 'UsagePrice'},
            'end': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '16', 'decimal_places': '6'}),
            'start': ('django.db.models.fields.DateField', [], {}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.UsageType']"})
        },
        'ralph_pricing.usagetype': {
            'Meta': {'object_name': 'UsageType'},
            'average': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'show_price_percentage': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'show_value_percentage': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'ralph_pricing.venture': {
            'Meta': {'object_name': 'Venture'},
            'business_segment': ('django.db.models.fields.TextField', [], {'default': "u''", 'max_length': '75', 'blank': 'True'}),
            'department': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '255'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'default': 'None', 'related_name': "u'children'", 'null': 'True', 'blank': 'True', 'to': "orm['ralph_pricing.Venture']"}),
            'profit_center': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '75', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'symbol': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '32', 'blank': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'venture_id': ('django.db.models.fields.IntegerField', [], {})
        }
    }

    complete_apps = ['ralph_pricing']


# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import decimal

from django.test import TestCase

from ralph_pricing.models import ExtraCostType, ExtraCost, Venture
from ralph_pricing.plugins.extra_cost import update_extra_cost


class TestExtraCostPlugin(TestCase):
    def setUp(self):
        self.date = datetime.date(2013, 01, 04)

    def get_extra_cost(self):
        """Simulated api result"""
        yield {
            'venture_id': 1,
            'venture': 'Venture1',
            'type': 'extracost_1',
            'cost': 100,
            'start': datetime.date(2013, 01, 01),
            'end': datetime.date(2013, 01, 05),
        }

    def test_sync_extra_cost(self):
        count = sum(
            update_extra_cost(data, self.date) for data in self.get_extra_cost()
        )
        self.assertEqual(count, 1)
        usage_type = ExtraCostType.objects.get(name='extracost_1')
        extra_cost = ExtraCost.objects.get(type=usage_type)
        venture = Venture.objects.get(name='Venture1')
        self.assertEqual(extra_cost.pricing_venture, venture)
        self.assertEqual(extra_cost.price, decimal.Decimal('3.278689'))



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import logging
import urllib

from django.conf import settings
from restkit import Resource, ResourceNotFound

from ralph.util import plugin
from ralph_pricing.models import UsageType, Venture, DailyUsage


logger = logging.getLogger(__name__)


def update_hamster_usage(venture, usage_type, date, url):
    venture_symbol = venture.symbol
    try:
        capacity = get_venture_capacity(venture_symbol, url)
    except ResourceNotFound:
        # apache not found error
        logger.error('Hamster data for %r not found' % venture_symbol)
    else:
        if capacity > 0:
            return set_hamster_usage(venture, capacity, usage_type, date)


def get_venture_capacity(venture_symbol, url):
    resource = Resource(url)
    ret = resource.get(urllib.quote_plus(venture_symbol.encode('utf-8')))
    json_data = json.loads(ret.body_string())
    return json_data.get('capacity')


def set_hamster_usage(venture, capacity, usage_type, date):
    usage, created = DailyUsage.objects.get_or_create(
        date=date,
        type=usage_type,
        pricing_venture=venture,
    )
    usage.value = capacity / (1024 * 1024)  # in MB
    usage.save()
    return created


@plugin.register(chain='pricing', requires=['ventures'])
def hamster(**kwargs):
    """Updates Hamster usage per Venture"""
    if not settings.HAMSTER_API_URL:
        return False, "Not configured", kwargs
    url = settings.HAMSTER_API_URL
    usage_type, created = UsageType.objects.get_or_create(
        name="Hamster Capacity 1 MB",
    )
    date = kwargs['today']
    ventures = Venture.objects.all()
    count = sum(
        update_hamster_usage(venture, usage_type, date, url) or 0
        for venture in ventures
    )
    return True, '%d new Hamster usage added in Ventures' % count, kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import mock

from django.conf import settings
from django.test import TestCase

from ralph_pricing.models import DailyUsage, Venture
from ralph_pricing.plugins.openstack import (
    openstack as openstack_runner,
)
from ralph_pricing.tests.samples.openstack import (
    simple_tenant_usage_data,
    tenants,
    tenants_data,
    tenants_usages_data,
)


class MockOpenStack(object):
    """ Simple mock for OpenStack network library """
    def __init__(self, *args, **kwargs):
        pass

    def __call__(self, *args, **kwargs):
        return self

    def simple_tenant_usage(self, start, end):
        return simple_tenant_usage_data

    def query(self, query, url, **kwargs):
        if query == 'os-simple-tenant-usage':
            result = tenants_usages_data
        elif query == 'simple-tenant-usage':
            result = simple_tenant_usage_data
        elif query == 'tenants':
            result = tenants_data
        return result

    def auth(self, *args, **kwargs):
        pass

    def get_ventures(self):
        return tenants

    def __getattr__(self, name):
        return mock.Mock()


class TestOpenstack(TestCase):
    def setUp(self):
        Venture(
            name='Test Venture1',
            symbol='test_venture1',
            venture_id='1'
        ).save()
        Venture(
            name='Test Venture2',
            symbol='test_venture2',
            venture_id='2'
        ).save()
        Venture(
            name='Test Venture3',
            symbol='test_venture3',
            venture_id='3'
        ).save()

    def test_set_usages(self):
        """ OpenStack usages Test Case """
         # fake setting need to run plugin
        settings.OPENSTACK_URL = '/'
        settings.OPENSTACK_USER = 'test'
        settings.OPENSTACK_PASSWORD = 'test'
        with mock.patch(
            'ralph_pricing.plugins.openstack.OpenStack'
        ) as OpenStack:
            OpenStack.side_effect = MockOpenStack
            status, message, arg = openstack_runner(
                today=datetime.datetime.today()
            )
            self.assertTrue(status)
            # usages venture1
            usage_venture1 = DailyUsage.objects.filter(
                pricing_venture__symbol='test_venture1',
            )
            self.assertEqual(len(usage_venture1), 5)
            memory_usage_venture1 = usage_venture1.get(
                type__name='OpenStack 10000 Memory GiB Hours'
            )
            volume_usage_venutre1 = usage_venture1.get(
                type__name='OpenStack 10000 Volume GiB Hours'
            )
            vcpus_usage_venutre1 = usage_venture1.get(
                type__name='OpenStack 10000 CPU Hours'
            )
            disk_usage_venutre1 = usage_venture1.get(
                type__name='OpenStack 10000 Disk GiB Hours'
            )
            images_usage_venutre1 = usage_venture1.get(
                type__name='OpenStack 10000 Images GiB Hours'
            )
            self.assertEqual(memory_usage_venture1.value, 393216.0 / 1024)
            self.assertEqual(volume_usage_venutre1.value, 480.0)
            self.assertEqual(vcpus_usage_venutre1.value, 192.0)
            self.assertEqual(disk_usage_venutre1.value, 768.0)
            self.assertEqual(images_usage_venutre1.value, 199.3)
            # usages venture2
            usage_venture2 = DailyUsage.objects.filter(
                pricing_venture__symbol='test_venture2',
            )
            self.assertEqual(len(usage_venture2), 5)
            memory_usage_venture2 = usage_venture2.get(
                type__name='OpenStack 10000 Memory GiB Hours'
            )
            volume_usage_venutre2 = usage_venture2.get(
                type__name='OpenStack 10000 Volume GiB Hours'
            )
            vcpus_usage_venutre2 = usage_venture2.get(
                type__name='OpenStack 10000 CPU Hours'
            )
            disk_usage_venutre2 = usage_venture2.get(
                type__name='OpenStack 10000 Disk GiB Hours'
            )
            images_usage_venutre2 = usage_venture2.get(
                type__name='OpenStack 10000 Images GiB Hours'
            )
            self.assertEqual(memory_usage_venture2.value, 786432.0 / 1024)
            self.assertEqual(volume_usage_venutre2.value, 28800.0)
            self.assertEqual(vcpus_usage_venutre2.value, 384.0)
            self.assertEqual(disk_usage_venutre2.value, 1536.0)
            self.assertEqual(images_usage_venutre2.value, 315.0)
            # no data for venture3
            usage_venture3 = DailyUsage.objects.filter(
                pricing_venture__symbol='test_venture3',
            )
            self.assertEqual(len(usage_venture3), 0)

    def test_fail_plugin(self):
        """ Testing not configured plugin """
        with mock.patch(
            'ralph_pricing.plugins.openstack.OpenStack'
        ) as OpenStack:
            OpenStack.side_effect = MockOpenStack
            status, message, arg = openstack_runner(
                today=datetime.datetime.today()
            )
            self.assertFalse(status)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime

from django.test import TestCase

from ralph_pricing import models
from ralph_pricing.views.ventures import AllVentures, TopVentures
from ralph.business.models import Venture


class TestVentures(TestCase):
    def setUp(self):
        ralphVenture = Venture(id=3, name='a', show_in_ralph=True)
        ralphVenture.save()
        ralphSubVenture = Venture(
            id=2,
            parent=ralphVenture,
            name='b',
            show_in_ralph=True,
        )
        ralphSubVenture.save()

    def test_ventures_all(self):
        day = datetime.date(2013, 4, 25)
        venture = models.Venture(venture_id=3, name='a')
        venture.save()
        device = models.Device(
            device_id=3,
            asset_id=5,
        )
        device.save()
        daily = models.DailyDevice(
            pricing_device=device,
            date=day,
            name='ziew',
            price='1337',
            pricing_venture=venture,
        )
        daily.save()
        subventure = models.Venture(venture_id=2, parent=venture, name='b')
        subventure.save()
        other_device = models.Device(
            device_id=2,
            asset_id=3,
        )
        other_device.save()
        other_daily = models.DailyDevice(
            pricing_device=other_device,
            date=day,
            name='ziew',
            price='833833',
            pricing_venture=subventure,
        )
        other_daily.save()
        usage_type = models.UsageType(name='waciki')
        usage_type.save()
        daily_usage = models.DailyUsage(
            type=usage_type,
            value=32,
            date=day,
            pricing_venture=venture,
        )
        daily_usage.save()
        extra_cost_type = models.ExtraCostType(name='waciki')
        extra_cost_type.save()
        extra_cost = models.ExtraCost(
            pricing_venture=venture,
            start=day,
            end=day,
            type=extra_cost_type,
            price='65535',
        )
        extra_cost.save()
        view = TopVentures()
        for progress, data in view.get_data(day, day, show_in_ralph=True):
            pass
        self.assertEquals(
            data,
            [
                [
                    3,  # id
                    'a',  # path
                    True, # show_in_ralph
                    '',  # department
                    '',  # business segment
                    '',  # Profit center
                    2.0,  # assets count
                    '835 170.00 PLN',  # assets price
                    '0.00 PLN',  # assets cost
                    32.0,  # usage count
                    '0.00 PLN',  # usage price
                    '65 535.00 PLN',  # extra cost
                ],
            ],
        )
        view = AllVentures()
        for progress, data in view.get_data(day, day, show_in_ralph=True):
            pass
        self.assertEquals(
            data,
            [
                [
                    3,
                    'a',
                    True, # show_in_ralph
                    '',
                    '',
                    '',
                    1.0,
                    '1 337.00 PLN',
                    '0.00 PLN',
                    32.0,
                    'NO PRICE',
                    '65 535.00 PLN',
                ],
                [
                    2,
                    'a/b',
                    True, # show_in_ralph
                    '',
                    '',
                    '',
                    1.0,
                    '833 833.00 PLN',
                    '0.00 PLN',
                    0,
                    '0.00 PLN',
                    '0.00 PLN',
                ],
            ],
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph.util import plugin, api_pricing
from ralph_pricing.models import UsageType, DailyUsage, Device, DailyDevice


def update_usage(device, venture, usage_name, date, value):
    if not value:
        return
    usage_type, created = UsageType.objects.get_or_create(
        name=usage_name
    )
    usage_type.average = True
    usage_type.save()
    usage, created = DailyUsage.objects.get_or_create(
        date=date,
        type=usage_type,
        pricing_device=device,
    )
    usage.pricing_venture = venture
    usage.value = value
    usage.save()


def update(data, date):
    if data.get('mount_device_id') is not None:
        device, created = Device.objects.get_or_create(
            device_id=data['mount_device_id'],
        )
    else:
        return
    try:
        daily_device = device.dailydevice_set.get(date=date)
        venture = daily_device.pricing_venture
    except DailyDevice.DoesNotExist:
        venture = None
    size = data['size'] / data['share_mount_count']
    usage_name = 'Disk Share {0} MB'.format(data['model'])
    update_usage(device, venture, usage_name, date, size)


@plugin.register(chain='pricing', requires=['devices'])
def shares(**kwargs):
    """Updates the disk share usages from Ralph."""

    date = kwargs['today']
    for data in api_pricing.get_shares():
        update(data, date)
    return True, 'disk share usages updated', kwargs



hosts_usages_data = [
    {
        u'MBytes': u'10318.234132',
        u'sourcetype': u'splunk_host1',
        u'percent': u'27.40',
        u'source': u'/var/log/apache2/error.log',
        u'host': u'test_host1',
        u'lastReceived': u'Thu 2013-05-16 06:09:09',
    },
    {
        u'MBytes': u'1326.640829',
        u'sourcetype': u'splunk_host2',
        u'percent': u'3.52',
        u'source': u'WinEventLog',
        u'host': u'test_host2',
        u'lastReceived': u'Fri 2013-05-17 07:26:16',
    },
    {
        u'MBytes': u'1048.363416',
        u'sourcetype': u'syslog',
        u'percent': u'2.78',
        u'source': u'syslog',
        u'host': u'test_host3',
        u'lastReceived': u'Fri 2013-05-17 07:27:02',
    },
]




# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph_pricing.views.base import Base


class Home(Base):
    template_name = 'ralph_pricing/home.html'

    def get_context_data(self, **kwargs):
        context = super(Home, self).get_context_data(**kwargs)
        context.update({
        })
        return context



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph.util import plugin, api_pricing
from ralph_pricing.models import Device, ParentDevice, Venture, DailyDevice


def update_device(data, date):
    device, created = Device.objects.get_or_create(
        device_id=data['id'],
    )
    device.name = data['name']
    device.sn = data['sn']
    device.barcode = data['barcode']
    device.is_virtual = data['is_virtual']
    device.is_blade = data['is_blade']
    device.save()

    daily, daily_created = DailyDevice.objects.get_or_create(
        date=date,
        pricing_device=device,
    )
    if data.get('parent_id'):
        parent, parent_created = ParentDevice.objects.get_or_create(
            device_id=data['parent_id'],
        )
        daily.parent = parent
    else:
        parent_created = False
    if data.get('venture_id') is not None:
        venture, venture_created = Venture.objects.get_or_create(
            venture_id=data['venture_id'],
        )
        daily.pricing_venture = venture
    daily.name = data['name']
    daily.save()
    return created + parent_created


@plugin.register(chain='pricing', requires=['ventures'])
def devices(**kwargs):
    """Updates the devices from Ralph."""

    date = kwargs['today']
    count = sum(
        update_device(data, date) for data in api_pricing.get_devices()
    )
    return True, '%d new devices' % count, kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.conf.urls.defaults import patterns, url
from django.contrib.auth.decorators import login_required

from ralph_pricing.views.devices import Devices
from ralph_pricing.views.extra_costs import ExtraCosts
from ralph_pricing.views.home import Home
from ralph_pricing.views.usages import Usages
from ralph_pricing.views.ventures import AllVentures, TopVentures


urlpatterns = patterns(
    '',
    url(r'^$', login_required(Home.as_view()), name='home'),
    url(
        r'^extra-costs/$',
        login_required(ExtraCosts.as_view()),
        name='extra_costs',
        kwargs={'venture': None},
    ),
    url(
        r'^extra-costs/(?P<venture>\d+)/$',
        login_required(ExtraCosts.as_view()),
        name='extra_costs',
    ),
    url(
        r'^usages/$',
        login_required(Usages.as_view()),
        name='usages',
        kwargs={'usage': None},
    ),
    url(
        r'^usages/(?P<usage>[^/]+)/$',
        login_required(Usages.as_view()),
        name='usages',
    ),
    url(
        r'^all-ventures/$',
        login_required(AllVentures.as_view()),
        name='all_ventures',
    ),
    url(
        r'^top-ventures/$',
        login_required(TopVentures.as_view()),
        name='top_ventures',
    ),
    url(
        r'^devices/$',
        login_required(Devices.as_view()),
        name='devices',
    ),
    url(
        r'^devices/(?P<venture>\d+)/$',
        login_required(Devices.as_view()),
        name='devices',
    ),
)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime

from django.test import TestCase

from ralph_pricing.models import Device, DailyDevice
from ralph_pricing.plugins.assets import update_assets


class TestAssetPlugin(TestCase):
    def setUp(self):
        self.today = datetime.date.today()

    def get_asset(self):
        """Simulated api result"""
        yield {
            'asset_id': 1123,
            'ralph_id': 13342,
            'slots': 10.0,
            'price': 100,
            'is_deprecated': True,
            'sn': '1234-1234-1234-1234',
            'barcode': '4321-4321-4321-4321',
            'deprecation_rate': 0,
        }

    def test_sync_asset_device(self):
        count = sum(
            update_assets(data, self.today) for data in self.get_asset()
        )
        self.assertEqual(count, 1)
        device = Device.objects.get(device_id=13342)
        self.assertEqual(device.device_id, 13342)
        self.assertEqual(device.asset_id, 1123)
        self.assertEqual(device.slots, 10.0)
        self.assertEqual(device.sn, '1234-1234-1234-1234')
        self.assertEqual(device.barcode, '4321-4321-4321-4321')

    def test_sync_asset_daily(self):
        count = sum(
            update_assets(data, self.today) for data in self.get_asset()
        )
        self.assertEqual(count, 1)
        daily = DailyDevice.objects.get(date=self.today)
        self.assertEqual(daily.is_deprecated, True)
        self.assertEqual(daily.price, 100)
        self.assertEqual(daily.pricing_device_id, 1)
        self.assertEqual(daily.date, self.today)

    def test_sync_asset_device_without_ralph_id(self):
        data = yield {
            'asset_id': 1123,
            'ralph_id': None,
            'slots': 10.0,
            'price': 100,
            'is_deprecated': True,
            'sn': '1234-1234-1234-1234',
            'barcode': '4321-4321-4321-4321',
            'deprecation_rate': 0,
        }
        count = sum(update_assets(item, self.today) for item in data)
        self.assertFalse(count > 0)

    def test_sync_asset_device_update(self):
        data = yield {
            'asset_id': 1123,
            'ralph_id': 123,
            'slots': 10.0,
            'price': 100,
            'is_deprecated': True,
            'sn': '1234-1234-1234-1234',
            'barcode': '4321-4321-4321-4321',
            'deprecation_rate': 0,
        }
        count = sum(update_assets(item, self.today) for item in data)
        self.assertFalse(count == 1)
        device = Device.objects.get(device_id=123)
        self.assertEqual(device.sn, '1234-1234-1234-1234')

        data = yield {
            'asset_id': 1123,
            'ralph_id': 123,
            'slots': 10.0,
            'price': 100,
            'is_deprecated': True,
            'sn': '5555-5555-5555-5555',
            'barcode': '4321-4321-4321-4321',
            'deprecation_rate': 0,
        }
        device = Device.objects.get(device_id=123)
        self.assertEqual(device.sn, '5555-5555-5555-5555')







# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import time

from django.conf import settings

from ralph.util import plugin
from ralph_pricing.splunk import Splunk
from ralph_pricing.models import (
    DailyDevice,
    DailyUsage,
    Device,
    SplunkName,
    UsageType,
    Venture,
)


def set_device_usage(date, usage, usage_type, device):
    try:
        daily_device = DailyDevice.objects.get(
            pricing_device=device,
            date=date,
        )
    except DailyDevice.DoesNotExist:
        venture = None
    else:
        venture = daily_device.pricing_venture
    daily_usage, created = DailyUsage.objects.get_or_create(
        date=date,
        type=usage_type,
        pricing_device=device,
        pricing_venture=venture,
    )
    daily_usage.value = usage
    daily_usage.save()


def set_unknown_usage(date, usage, usage_type, splunk_venture):
    daily_usage, created = DailyUsage.objects.get_or_create(
        date=date,
        type=usage_type,
        pricing_venture=splunk_venture,
        value=usage,
    )


def set_usages(date, usage, usage_type, host, splunk_venture):
    try:
        splunk_pair = SplunkName.objects.get(
            splunk_name=host,
            pricing_device__name=host,
        )
    except SplunkName.DoesNotExist:
        try:
            splunk_pair = SplunkName.objects.get(
                splunk_name=host,
                pricing_device__isnull=False,
            )
        except SplunkName.DoesNotExist:
            device = Device.objects.filter(name=host)
            if device:
                splunk_pair = SplunkName(
                    splunk_name=host,
                    pricing_device=device[0],
                )
                splunk_pair.save()
                set_device_usage(date, usage, usage_type, device[0])
            else:
                SplunkName.objects.get_or_create(splunk_name=host)
                set_unknown_usage(date, usage, usage_type, splunk_venture)
        else:
            set_device_usage(
                date, usage, usage_type, splunk_pair.pricing_device,
            )
    else:
        set_device_usage(date, usage, usage_type, splunk_pair.pricing_device)


@plugin.register(chain='pricing', requires=['ventures', 'devices'])
def splunk(**kwargs):
    """Updates Splunk usage per Venture"""
    if not settings.SPLUNK_HOST:
        return False, "Not configured.", kwargs
    try:
        splunk_venture = Venture.objects.get(symbol='splunk_unknown_usage')
    except Venture.DoesNotExist:
        return False, 'Splunk venture does not exist!', kwargs
    usage_type, created = UsageType.objects.get_or_create(
        name="Splunk Volume 1 MB",
    )
    date = kwargs['today']
    splunk = Splunk()
    days_ago = (date - datetime.date.today()).days
    earliest = '{}d@d'.format(days_ago - 1)
    latest = '{}d@d'.format(days_ago) if days_ago != 0 else 'now'
    splunk.start(earliest=earliest, latest=latest)
    percent = splunk.progress
    while percent < 100:
        print(percent)
        time.sleep(30)
        percent = splunk.progress
    hosts = {}
    for item in splunk.results:
        host = item['host']
        mb = float(item['MBytes'])
        if host in hosts:
            hosts[host] += mb
        else:
            hosts[host] = mb
    for host, usage in hosts.iteritems():
        set_usages(date, usage, usage_type, host, splunk_venture)
    return True, 'done.', kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import decimal

from ralph.util import plugin, api_pricing
from ralph_pricing.models import ExtraCost, ExtraCostType, Venture


def update_extra_cost(data, date):
    if not data['cost']:
        return False  # False = 0 in sum function
    cost_type, created = ExtraCostType.objects.get_or_create(name=data['type'])
    venture, created = Venture.objects.get_or_create(
        venture_id=data['venture_id'],
        defaults={'name': data['venture']},
    )
    extracost, created = ExtraCost.objects.get_or_create(
        start=data['start'],
        end=data['end'] if data['end'] else datetime.date(2048, 10, 24),
        type=cost_type,
        pricing_venture=venture,
        defaults={'price': decimal.Decimal('0')},
    )
    extracost.price = decimal.Decimal(data['cost'] / 30.5)
    extracost.save()
    return created


@plugin.register(chain='pricing', requires=['never run'])
def extracost(**kwargs):
    date = kwargs['today']
    count = sum(
        update_extra_cost(data, date) for data in api_pricing.get_extra_cost()
    )
    return True, '%d new extracosts' % count, kwargs



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Device'
        db.create_table('ralph_pricing_device', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('device_id', self.gf('django.db.models.fields.IntegerField')(unique=True)),
            ('asset_id', self.gf('django.db.models.fields.IntegerField')(default=None, unique=True, null=True, blank=True)),
            ('is_virtual', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_blade', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('slots', self.gf('django.db.models.fields.FloatField')(default=0)),
        ))
        db.send_create_signal('ralph_pricing', ['Device'])

        # Adding model 'Venture'
        db.create_table('ralph_pricing_venture', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('venture_id', self.gf('django.db.models.fields.IntegerField')()),
            ('name', self.gf('django.db.models.fields.CharField')(default=u'', max_length=255)),
            ('department', self.gf('django.db.models.fields.CharField')(default=u'', max_length=255)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(default=None, related_name=u'children', null=True, blank=True, to=orm['ralph_pricing.Venture'])),
            ('lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            ('level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal('ralph_pricing', ['Venture'])

        # Adding model 'DailyPart'
        db.create_table('ralph_pricing_dailypart', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date', self.gf('django.db.models.fields.DateField')()),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('pricing_device', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ralph_pricing.Device'])),
            ('asset_id', self.gf('django.db.models.fields.IntegerField')()),
            ('price', self.gf('django.db.models.fields.DecimalField')(max_digits=16, decimal_places=6)),
            ('is_deprecated', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('ralph_pricing', ['DailyPart'])

        # Adding unique constraint on 'DailyPart', fields ['date', 'asset_id']
        db.create_unique('ralph_pricing_dailypart', ['date', 'asset_id'])

        # Adding model 'DailyDevice'
        db.create_table('ralph_pricing_dailydevice', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date', self.gf('django.db.models.fields.DateField')()),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('pricing_device', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ralph_pricing.Device'])),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(related_name=u'child_set', on_delete=models.SET_NULL, default=None, to=orm['ralph_pricing.Device'], blank=True, null=True)),
            ('price', self.gf('django.db.models.fields.DecimalField')(default=0, max_digits=16, decimal_places=6)),
            ('pricing_venture', self.gf('django.db.models.fields.related.ForeignKey')(default=None, to=orm['ralph_pricing.Venture'], null=True, on_delete=models.SET_NULL, blank=True)),
            ('is_deprecated', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('ralph_pricing', ['DailyDevice'])

        # Adding unique constraint on 'DailyDevice', fields ['date', 'pricing_device']
        db.create_unique('ralph_pricing_dailydevice', ['date', 'pricing_device_id'])

        # Adding model 'UsageType'
        db.create_table('ralph_pricing_usagetype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('ralph_pricing', ['UsageType'])

        # Adding model 'UsagePrice'
        db.create_table('ralph_pricing_usageprice', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ralph_pricing.UsageType'])),
            ('price', self.gf('django.db.models.fields.DecimalField')(max_digits=16, decimal_places=6)),
            ('start', self.gf('django.db.models.fields.DateField')()),
            ('end', self.gf('django.db.models.fields.DateField')()),
        ))
        db.send_create_signal('ralph_pricing', ['UsagePrice'])

        # Adding unique constraint on 'UsagePrice', fields ['start', 'type']
        db.create_unique('ralph_pricing_usageprice', ['start', 'type_id'])

        # Adding unique constraint on 'UsagePrice', fields ['end', 'type']
        db.create_unique('ralph_pricing_usageprice', ['end', 'type_id'])

        # Adding model 'DailyUsage'
        db.create_table('ralph_pricing_dailyusage', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date', self.gf('django.db.models.fields.DateField')()),
            ('pricing_venture', self.gf('django.db.models.fields.related.ForeignKey')(default=None, to=orm['ralph_pricing.Venture'], null=True, on_delete=models.SET_NULL, blank=True)),
            ('pricing_device', self.gf('django.db.models.fields.related.ForeignKey')(default=None, to=orm['ralph_pricing.Device'], null=True, on_delete=models.SET_NULL, blank=True)),
            ('value', self.gf('django.db.models.fields.FloatField')()),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ralph_pricing.UsageType'])),
        ))
        db.send_create_signal('ralph_pricing', ['DailyUsage'])

        # Adding unique constraint on 'DailyUsage', fields ['date', 'pricing_device', 'type']
        db.create_unique('ralph_pricing_dailyusage', ['date', 'pricing_device_id', 'type_id'])

        # Adding model 'ExtraCostType'
        db.create_table('ralph_pricing_extracosttype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('ralph_pricing', ['ExtraCostType'])

        # Adding model 'ExtraCost'
        db.create_table('ralph_pricing_extracost', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('start', self.gf('django.db.models.fields.DateField')()),
            ('end', self.gf('django.db.models.fields.DateField')()),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ralph_pricing.ExtraCostType'])),
            ('price', self.gf('django.db.models.fields.DecimalField')(max_digits=16, decimal_places=6)),
            ('pricing_venture', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['ralph_pricing.Venture'])),
        ))
        db.send_create_signal('ralph_pricing', ['ExtraCost'])

        # Adding unique constraint on 'ExtraCost', fields ['start', 'pricing_venture', 'type']
        db.create_unique('ralph_pricing_extracost', ['start', 'pricing_venture_id', 'type_id'])

        # Adding unique constraint on 'ExtraCost', fields ['end', 'pricing_venture', 'type']
        db.create_unique('ralph_pricing_extracost', ['end', 'pricing_venture_id', 'type_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'ExtraCost', fields ['end', 'pricing_venture', 'type']
        db.delete_unique('ralph_pricing_extracost', ['end', 'pricing_venture_id', 'type_id'])

        # Removing unique constraint on 'ExtraCost', fields ['start', 'pricing_venture', 'type']
        db.delete_unique('ralph_pricing_extracost', ['start', 'pricing_venture_id', 'type_id'])

        # Removing unique constraint on 'DailyUsage', fields ['date', 'pricing_device', 'type']
        db.delete_unique('ralph_pricing_dailyusage', ['date', 'pricing_device_id', 'type_id'])

        # Removing unique constraint on 'UsagePrice', fields ['end', 'type']
        db.delete_unique('ralph_pricing_usageprice', ['end', 'type_id'])

        # Removing unique constraint on 'UsagePrice', fields ['start', 'type']
        db.delete_unique('ralph_pricing_usageprice', ['start', 'type_id'])

        # Removing unique constraint on 'DailyDevice', fields ['date', 'pricing_device']
        db.delete_unique('ralph_pricing_dailydevice', ['date', 'pricing_device_id'])

        # Removing unique constraint on 'DailyPart', fields ['date', 'asset_id']
        db.delete_unique('ralph_pricing_dailypart', ['date', 'asset_id'])

        # Deleting model 'Device'
        db.delete_table('ralph_pricing_device')

        # Deleting model 'Venture'
        db.delete_table('ralph_pricing_venture')

        # Deleting model 'DailyPart'
        db.delete_table('ralph_pricing_dailypart')

        # Deleting model 'DailyDevice'
        db.delete_table('ralph_pricing_dailydevice')

        # Deleting model 'UsageType'
        db.delete_table('ralph_pricing_usagetype')

        # Deleting model 'UsagePrice'
        db.delete_table('ralph_pricing_usageprice')

        # Deleting model 'DailyUsage'
        db.delete_table('ralph_pricing_dailyusage')

        # Deleting model 'ExtraCostType'
        db.delete_table('ralph_pricing_extracosttype')

        # Deleting model 'ExtraCost'
        db.delete_table('ralph_pricing_extracost')


    models = {
        'ralph_pricing.dailydevice': {
            'Meta': {'ordering': "(u'pricing_device', u'date')", 'unique_together': "((u'date', u'pricing_device'),)", 'object_name': 'DailyDevice'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deprecated': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'child_set'", 'on_delete': 'models.SET_NULL', 'default': 'None', 'to': "orm['ralph_pricing.Device']", 'blank': 'True', 'null': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '16', 'decimal_places': '6'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.Device']"}),
            'pricing_venture': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Venture']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'})
        },
        'ralph_pricing.dailypart': {
            'Meta': {'ordering': "(u'asset_id', u'pricing_device', u'date')", 'unique_together': "((u'date', u'asset_id'),)", 'object_name': 'DailyPart'},
            'asset_id': ('django.db.models.fields.IntegerField', [], {}),
            'date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deprecated': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '16', 'decimal_places': '6'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.Device']"})
        },
        'ralph_pricing.dailyusage': {
            'Meta': {'ordering': "(u'pricing_device', u'type', u'date')", 'unique_together': "((u'date', u'pricing_device', u'type'),)", 'object_name': 'DailyUsage'},
            'date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pricing_device': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Device']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'pricing_venture': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['ralph_pricing.Venture']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.UsageType']"}),
            'value': ('django.db.models.fields.FloatField', [], {})
        },
        'ralph_pricing.device': {
            'Meta': {'object_name': 'Device'},
            'asset_id': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'device_id': ('django.db.models.fields.IntegerField', [], {'unique': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_blade': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_virtual': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slots': ('django.db.models.fields.FloatField', [], {'default': '0'})
        },
        'ralph_pricing.extracost': {
            'Meta': {'unique_together': "[(u'start', u'pricing_venture', u'type'), (u'end', u'pricing_venture', u'type')]", 'object_name': 'ExtraCost'},
            'end': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '16', 'decimal_places': '6'}),
            'pricing_venture': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.Venture']"}),
            'start': ('django.db.models.fields.DateField', [], {}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.ExtraCostType']"})
        },
        'ralph_pricing.extracosttype': {
            'Meta': {'object_name': 'ExtraCostType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'ralph_pricing.usageprice': {
            'Meta': {'ordering': "(u'type', u'start')", 'unique_together': "[(u'start', u'type'), (u'end', u'type')]", 'object_name': 'UsagePrice'},
            'end': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'price': ('django.db.models.fields.DecimalField', [], {'max_digits': '16', 'decimal_places': '6'}),
            'start': ('django.db.models.fields.DateField', [], {}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['ralph_pricing.UsageType']"})
        },
        'ralph_pricing.usagetype': {
            'Meta': {'object_name': 'UsageType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'ralph_pricing.venture': {
            'Meta': {'object_name': 'Venture'},
            'department': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'default': "u''", 'max_length': '255'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'default': 'None', 'related_name': "u'children'", 'null': 'True', 'blank': 'True', 'to': "orm['ralph_pricing.Venture']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'venture_id': ('django.db.models.fields.IntegerField', [], {})
        }
    }

    complete_apps = ['ralph_pricing']


# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import collections
import datetime

from django import forms
from django.utils.translation import ugettext_lazy as _

from mptt.forms import TreeNodeChoiceField

from ralph.ui.widgets import DateWidget
from ralph_pricing.models import ExtraCost, UsagePrice, Venture


class ExtraCostForm(forms.ModelForm):
    class Meta:
        model = ExtraCost
        fields = 'type', 'price', 'start', 'end'
        widgets = {
            'start': DateWidget(attrs={'class': 'input-small'}),
            'end': DateWidget(attrs={'class': 'input-small'}),
        }

    def clean_end(self):
        start = self.cleaned_data['start']
        end = self.cleaned_data['end']
        if start > end:
            raise forms.ValidationError(
                _("End date must be later than or equal to the start date."),
            )
        return end


class ExtraCostBaseFormSet(forms.models.BaseModelFormSet):
    def clean(self):
        if any(self.errors):
            return
        types = collections.defaultdict(list)
        for i in xrange(self.total_form_count()):
            form = self.forms[i]
            type_ = form.cleaned_data.get('type')
            start = form.cleaned_data.get('start')
            end = form.cleaned_data.get('end')
            if not type_ or not start or not end:
                continue
            for other_start, other_end in types[type_]:
                if other_start <= start <= other_end:
                    form._errors['start'] = form.error_class([
                        _("Start date overlaps with an existing extra "
                            "cost of the same type."),
                    ])
                    break
                if other_start <= end <= other_end:
                    form._errors['end'] = form.error_class([
                        _("End date overlaps with an existing extra "
                            "cost of the same type."),
                    ])
                    break
                if start <= other_start <= end:
                    form._errors['start'] = form.error_class([
                        _("A start date of an existing extra cost of "
                            "the same type overlaps with this time span."),
                    ])
                    break
                if start <= other_end <= end:
                    form._errors['start'] = form.error_class([
                        _("An end date of an existing extra cost of "
                            "the same type overlaps with this time span."),
                    ])
                    break
            else:
                types[type_].append((start, end))


ExtraCostFormSet = forms.models.modelformset_factory(
    ExtraCost,
    form=ExtraCostForm,
    formset=ExtraCostBaseFormSet,
    can_delete=True,
)


class UsagePriceForm(forms.ModelForm):
    class Meta:
        model = UsagePrice
        fields = 'price', 'start', 'end'
        widgets = {
            'start': DateWidget(attrs={'class': 'input-small'}),
            'end': DateWidget(attrs={'class': 'input-small'}),
        }

    def clean_end(self):
        start = self.cleaned_data['start']
        end = self.cleaned_data['end']
        if start > end:
            raise forms.ValidationError(
                _("End date must be later than or equal to the start date."),
            )
        return end


class UsagesBaseFormSet(forms.models.BaseModelFormSet):
    def clean(self):
        if any(self.errors):
            return
        dates = []
        for i in xrange(self.total_form_count()):
            form = self.forms[i]
            start = form.cleaned_data.get('start')
            end = form.cleaned_data.get('end')
            if not start or not end:
                continue
            for other_start, other_end in dates:
                if other_start <= start <= other_end:
                    form._errors['start'] = form.error_class([
                        _("Start date overlaps with an existing extra "
                            "cost of the same type."),
                    ])
                    break
                if other_start <= end <= other_end:
                    form._errors['end'] = form.error_class([
                        _("End date overlaps with an existing extra "
                            "cost of the same type."),
                    ])
                    break
                if start <= other_start <= end:
                    form._errors['start'] = form.error_class([
                        _("A start date of an existing extra cost of "
                            "the same type overlaps with this time span."),
                    ])
                    break
                if start <= other_end <= end:
                    form._errors['start'] = form.error_class([
                        _("An end date of an existing extra cost of "
                            "the same type overlaps with this time span."),
                    ])
                    break
            else:
                dates.append((start, end))

UsagesFormSet = forms.models.modelformset_factory(
    UsagePrice,
    form=UsagePriceForm,
    formset=UsagesBaseFormSet,
    can_delete=True,
)


class DateRangeForm(forms.Form):
    start = forms.DateField(
        widget=DateWidget(
            attrs={'class': 'input-small'},
        ),
        label='',
        initial=lambda: datetime.date.today() - datetime.timedelta(days=30),
    )
    end = forms.DateField(
        widget=DateWidget(
            attrs={'class': 'input-small'},
        ),
        label='',
        initial=datetime.date.today,
    )
    show_in_ralph = forms.BooleanField(
        required=False,
        label=_("Show only active"),
    )


class DateRangeVentureForm(DateRangeForm):
    venture = TreeNodeChoiceField(
        queryset=Venture.tree.all(),
        level_indicator='|---',
        empty_label="---",
    )



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


from bob.menu import MenuItem
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.views.generic import TemplateView


from ralph_pricing import VERSION


MAIN_MENU = [
    MenuItem(
        _("All ventures"),
        name='all-ventures',
        fugue_icon='fugue-store-medium',
        view_name='all_ventures',
    ),
    MenuItem(
        _("Top ventures"),
        name='top-ventures',
        fugue_icon='fugue-store',
        view_name='top_ventures',
    ),
    MenuItem(
        _("Devices"),
        name='devices',
        fugue_icon='fugue-wooden-box',
        view_name='devices',
    ),
    MenuItem(
        _("Extra costs"),
        name='extra-costs',
        fugue_icon='fugue-money-coin',
        view_name='extra_costs',
    ),
    MenuItem(
        _("Usage types"),
        name='usages',
        fugue_icon='fugue-beaker',
        view_name='usages',
    ),
]


class Base(TemplateView):
    def get_context_data(self, **kwargs):
        context = super(Base, self).get_context_data(**kwargs)
        footer_items = []
        if self.request.user.is_staff:
            footer_items.append(
                MenuItem(
                    'Admin',
                    fugue_icon='fugue-toolbox',
                    href='/admin/ralph_pricing',
                ),
            )
        footer_items.append(
            MenuItem(
                'logout',
                fugue_icon='fugue-door-open-out',
                pull_right=True,
                href=settings.LOGOUT_URL,
            )
        )
        footer_items.append(
            MenuItem(
                'Ralph',
                fugue_icon='fugue-home',
                href=reverse('search'),
            )
        )
        footer_items.append(
            MenuItem(
                "Version %s" % '.'.join((str(part) for part in VERSION)),
                fugue_icon='fugue-document-number',
            )
        )

        context.update({
            'mainmenu_items': MAIN_MENU,
            'footer_items': footer_items,
            'home_url': reverse('home'),
        })
        return context



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import itertools
import urllib

from django.conf import settings
from django.core.cache import get_cache

from ralph_pricing.views.base import Base
from bob.csvutil import make_csv_response
import django_rq
from rq.job import Job
from django.contrib import messages
from django.core.cache.backends.dummy import DummyCache


CACHE_NAME = 'reports_pricing'
if CACHE_NAME not in settings.CACHES:
    CACHE_NAME = 'default'
QUEUE_NAME = 'reports_pricing'
if QUEUE_NAME not in settings.RQ_QUEUES:
    QUEUE_NAME = None
TIMEOUT = getattr(settings, 'PRICING_REPORTS_TIMEOUT', 4 * 3600)  # 4 hours


def currency(value):
    """Formats currency as string according to the settings."""

    return '{:,.2f} {}'.format(value or 0, settings.CURRENCY).replace(',', ' ')


def _get_cache_key(section, **kwargs):
    return b'{}?{}'.format(section, urllib.urlencode(kwargs))


class Report(Base):
    """
    A base class for the reports. Override ``template_name``, ``Form``,
    ``section``, ``get_header`` and ``get_data`` in the specific reports.

    Make sure that ``get_header`` and ``get_data`` are static methods.
    """
    template_name = None
    Form = None
    section = ''
    report_name = ''

    def __init__(self, *args, **kwargs):
        super(Report, self).__init__(*args, **kwargs)
        self.data = []
        self.header = []
        self.form = None
        self.progress = 0
        self.got_query = False

    def get(self, *args, **kwargs):
        get = self.request.GET
        if get:
            self.form = self.Form(get)
            self.got_query = True
        else:
            self.form = self.Form()
        if self.form.is_valid():
            if 'clear' in get:
                self.progress = 0
                self.got_query = False
                self._clear_cache(**self.form.cleaned_data)
                messages.success(
                    self.request, "Cache cleared for this report.",
                )
            else:
                self.progress, self.header, self.data = self._get_cached(
                    **self.form.cleaned_data
                )
                if get.get('format', '').lower() == 'csv':
                    if self.progress == 100:
                        return make_csv_response(
                            itertools.chain([self.header], self.data),
                            '{}.csv'.format(self.section),
                        )
                    else:
                        messages.warning(
                            self.request,
                            "Please wait for the report "
                            "to finish calculating.",
                        )
        return super(Report, self).get(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(Report, self).get_context_data(**kwargs)
        context.update({
            'progress': self.progress,
            'data': self.data,
            'header': self.header,
            'section': self.section,
            'report_name': self.report_name,
            'form': self.form,
            'got_query': self.got_query,
        })
        return context

    def _clear_cache(self, **kwargs):
        cache = get_cache(CACHE_NAME)
        key = _get_cache_key(self.section, **kwargs)
        cache.set(key, None)

    def _get_cached(self, **kwargs):
        cache = get_cache(CACHE_NAME)
        if isinstance(cache, DummyCache):
            # No caching or queues with dummy cache.
            header, data = self._get_header_and_data(**kwargs)
            return 100, header, data
        key = _get_cache_key(self.section, **kwargs)
        cached = cache.get(key)
        if cached is not None:
            progress, job_id, header, data = cached
            if progress < 100 and job_id is not None and QUEUE_NAME:
                connection = django_rq.get_connection(QUEUE_NAME)
                job = Job.fetch(job_id, connection)
                if job.is_finished:
                    header, data = job.result
                    progress = 100
                    cache.set(key, (progress, job_id, header, data))
                elif job.is_failed:
                    header, data = None, None
                    progress = 100
                    cache.delete(key)
        else:
            if QUEUE_NAME:
                queue = django_rq.get_queue(QUEUE_NAME)
                job = queue.enqueue_call(
                    func=self._get_header_and_data,
                    kwargs=kwargs,
                    timeout=TIMEOUT,
                )
                progress = 0
                header = None
                data = None
                cache.set(key, (progress, job.id, header, data))
            else:
                progress = 0
                cache.set(key, (progress, None, None, None))
                header, data = self._get_header_and_data(**kwargs)
                progress = 100
                cache.set(key, (progress, None, header, data))
        return progress, header or [], data or []

    @classmethod
    def _get_header_and_data(cls, **kwargs):
        cache = get_cache(CACHE_NAME)
        key = _get_cache_key(cls.section, **kwargs)
        cached = cache.get(key)
        if cached is not None:
            job_id = cached[1]
        else:
            job_id = None
        header = cls.get_header(**kwargs)
        last_progress = 0
        data = []
        for progress, data in cls.get_data(**kwargs):
            if job_id is not None and progress - last_progress > 5:
                # Update progress in 5% increments
                cache.set(key, (progress, job_id, header, data))
                last_progress = progress
        return header, data

    @staticmethod
    def get_data(**kwargs):
        """
        Override this static method to provide data for the report.
        It gets called with the form's data as arguments.
        Make sure it's a static method.
        """
        return []

    @staticmethod
    def get_header(**kwargs):
        """
        Override this static method to provide header for the report.
        It gets called with the form's data as arguments.
        Make sure it's a static method.
        """
        return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json

from django.conf import settings
from splunklib.client import connect


class Splunk(object):
    """Usage:

    >>> import time
    >>> splunk = Splunk('splunk.host', 'splunk.user', '********')
    >>> splunk.start(earliest='-60m')
    >>> while splunk.progress < 100:
    ...    time.sleep(60)
    >>> splunk.results
    [{u'MBytes': u'123.386241', u'sourcetype': u'splunk_kp',
     u'percent': u'5.48', u'source': u'/opt/kp/log/csFront.log',
     u'host': u'hostname1', u'lastReceived':u'Thu 2012-06-07 12:00:34'},
     {u'MBytes': u'40.942306', u'sourcetype': u'splunk_istore',
     u'percent': u'1.82', u'source': u'/tmp/istoreapi_webapi_traffic.log',
     u'host': u'hostname2', u'lastReceived': u'Thu 2012-06-07 12:00:34'}, ...]
    """

    query = ('search earliest=%s latest=%s index="_internal" '
             'source="*license_usage.log" | eval lastReceived = _time | '
             'rename h as host b as bytes st as sourcetype s as source | '
             'stats sum(bytes) as bytes max(lastReceived) as lastReceived '
             'by host sourcetype source | eval mbytes=((bytes/1024)/1024) | '
             'fields host sourcetype source mbytes lastReceived | '
             'stats max(lastReceived) as lastReceived sum(mbytes) as MBytes '
             'by host sourcetype source| fieldformat lastReceived = '
             'strftime(lastReceived, "%%a %%F %%T") | eventstats sum(MBytes) '
             'as totalMB | eval percent = round(100*MBytes/totalMB,2) | '
             'fields - totalMB | sort -MBytes')

    def __init__(
        self,
        host=settings.SPLUNK_HOST,
        username=settings.SPLUNK_USER,
        password=settings.SPLUNK_PASSWORD
    ):
        self.host = str(host)  # sic, unicode fails in the splunk-sdk
        self.username = str(username)
        self.password = str(password)
        self.splunk = connect(
            host=self.host,
            username=self.username,
            password=self.password,
        )
        self.job = None
        self._results = None

    def start(self, earliest='-1d@d', latest='now'):
        if self.job:
            raise ValueError("Report in progress.")
        self.job = self.splunk.jobs.create(self.query % (earliest, latest))
        self._results = None

    @property
    def progress(self):
        """Returns a float with the percent of the report finished. 100.0 means
        report complete."""

        if self._results:
            return 100.0
        if not self.job:
            raise ValueError("No report in progress.")
        stats = self.job.refresh()(
            'isDone',
            'doneProgress',
            'scanCount',
            'eventCount',
            'resultCount',
        )
        progress = float(stats['doneProgress']) * 100
        if progress > 99.9:
            if stats['isDone'] == '1':
                progress = 100.0
                self._results = self.job.results(
                    output_mode='json',
                    count=0,
                ).read()
                self.job = None
            else:
                progress = 99.9
        return progress

    @property
    def results(self):
        """Returns a list of results for each host per source-type."""
        if not self._results and self.progress < 100.0:
            raise ValueError("Report still in progress.")
        return json.loads(self._results)



# -*- encoding: utf-8 -*-

import os
import sys
from setuptools import setup, find_packages

assert sys.version_info >= (2, 7), "Python 2.7+ required."

current_dir = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(current_dir, 'README.rst')) as readme_file:
    with open(os.path.join(current_dir, 'CHANGES.rst')) as changes_file:
        long_description = readme_file.read() + '\n' + changes_file.read()

sys.path.insert(0, current_dir + os.sep + 'src')
from ralph_pricing import VERSION
release = ".".join(str(num) for num in VERSION)

setup(
    name='ralph_pricing',
    version=release,
    author='Grupa Allegro Sp. z o.o. and Contributors',
    author_email='it-ralph-dev@allegro.pl',
    description="Pricing module for Ralph",
    long_description=long_description,
    url='http://ralph.allegrogroup.com/',
    keywords='',
    platforms=['any'],
    license='Apache Software License v2.0',
    packages=find_packages('src'),
    include_package_data=True,
    package_dir={'': 'src'},
    zip_safe=False,  # because templates are loaded from file path
    install_requires=[
        'ralph>=1.2.2',
        'ralph_assets>=1.0.0',
        'django-mptt==0.5.5',
    ],
    entry_points={
        'console_scripts': [
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows NT/2000',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ralph.util import plugin, api_pricing
from ralph_pricing.models import UsageType, DailyUsage, Device, Venture


def update_cores(data, usage_type, date):
    if data.get('device_id') is not None:
        device, created = Device.objects.get_or_create(
            device_id=data['device_id'],
        )
    else:
        return
    usage, created = DailyUsage.objects.get_or_create(
        date=date,
        type=usage_type,
        pricing_device=device,
    )
    if data.get('venture_id') is not None:
        venture, created = Venture.objects.get_or_create(
            venture_id=data['venture_id'],
        )
        usage.pricing_venture = venture
    usage.value = data['physical_cores']
    usage.save()
    return usage.value

def get_usage():
    usage_type, created = UsageType.objects.get_or_create(
        name="Physical CPU cores",
    )
    usage_type.average = True
    usage_type.save()
    return usage_type


@plugin.register(chain='pricing', requires=['devices'])
def physical_cores(**kwargs):
    """Updates the physical cores from Ralph."""

    date = kwargs['today']
    usage = get_usage()
    count = sum(
        update_cores(data, usage, date)
        for data in api_pricing.get_physical_cores()
    )
    return True, '%d total physical cores' % count, kwargs



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import itertools

from django.utils.translation import ugettext_lazy as _

from ralph_pricing.forms import DateRangeVentureForm
from ralph_pricing.models import DailyDevice, Device
from ralph_pricing.views.reports import Report, currency


class Devices(Report):
    template_name = 'ralph_pricing/devices.html'
    Form = DateRangeVentureForm
    section = 'devices'
    report_name = _('Devices Report')

    @staticmethod
    def get_data(start, end, venture, **kwargs):
        if not venture:
            return
        devices_ids = DailyDevice.objects.filter(
            date__gte=start,
            date__lte=end,
            pricing_venture=venture,
        ).values_list('pricing_device_id', flat=True).distinct()
        total_count = len(devices_ids)
        devices = Device.objects.filter(id__in=devices_ids)
        data = []
        for extracost in venture.get_extracost_details(start, end):
            row = [
                '{} (Extra Cost)'.format(extracost.type.name),
                '',
                '',
                '{} - {}'.format(extracost.start, extracost.end),
                '',
                currency(extracost.price),
                '',
                '',
                '',
                '',
            ]
            data.append(row)
        for usage in venture.get_daily_usages(start, end):
            row = [
                '',
                '',
                usage['name'],
                '',
                '',
                '',
                '',
                currency(usage['price']),
                usage['count'],
            ]
            data.append(row)
        for i, device in enumerate(devices):
            count, price, cost = venture.get_assets_count_price_cost(
                start,
                end,
                device_id=device.id,
            )
            data.append([
                device.name,
                '',
                '',
                device.sn,
                device.barcode,
                device.get_deprecated_status(start, end, venture),
                currency(price),
                currency(cost),
                '',
            ])
            for part in device.get_daily_parts(start, end):
                data.append([
                    '',
                    part['name'],
                    '',
                    '',
                    '',
                    part.get('deprecation', ''),
                    currency(part['price']),
                    currency(part['cost']),
                    '',
                ])

            for usage in device.get_daily_usages(start, end):
                data.append([
                    '',
                    '',
                    usage['name'],
                    '',
                    '',
                    '',
                    '',
                    currency(usage['price']),
                    usage['count'],
                ])
            progress = (100 * i) // total_count
            yield progress, data

    @staticmethod
    def get_header(**kwargs):
        header = [
            _("Device"),
            _("Component name"),
            _("Usage name"),
            _("SN"),
            _("Barcode"),
            _("Deprecation"),
            _("Price"),
            _("Cost"),
            _("Usage count"),
        ]
        return header



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import mock

from django.conf import settings
from django.test import TestCase

from ralph_pricing.models import DailyUsage, Venture
from ralph_pricing.plugins.hamster import (
    hamster as hamster_runner,
)


def mock_get_venture_capacity(venture_symbol, url):
    if venture_symbol == 'test_venture1':
        capacity = 2131231233.0
    elif venture_symbol == 'test_venture2':
        capacity = 4234233423.0
    else:
        capacity = 0
    return capacity


class TestHamster(TestCase):
    def setUp(self):
        self.venture_1 = Venture(
            name='Test Venture1',
            symbol='test_venture1',
            venture_id='1',
        )
        self.venture_1.save()
        self.venture_2 = Venture(
            name='Test Venture2',
            symbol='test_venture2',
            venture_id='2',
        )
        self.venture_2.save()
        self.venture_3 = Venture(
            name='Test Venture3',
            symbol='test_venture3',
            venture_id='3',
        )
        self.venture_3.save()

    def test_set_usages(self):
        """ Hamster usages Test Case """
        # fake setting need to run plugin
        settings.HAMSTER_API_URL = "/"
        with mock.patch(
            'ralph_pricing.plugins.hamster.get_venture_capacity'
        ) as get_venture_capacity:
            get_venture_capacity.side_effect = mock_get_venture_capacity
            status, message, args = hamster_runner(
                today=datetime.datetime.today()
            )
            self.assertTrue(status)

            usages = DailyUsage.objects.all()
            self.assertEqual(len(usages), 2)

            usage_venture1 = DailyUsage.objects.get(
                pricing_venture=self.venture_1
            )
            usage_venture2 = DailyUsage.objects.get(
                pricing_venture=self.venture_2
            )
            self.assertEqual(
                usage_venture1.value, 2131231233.0 / (1024 * 1024)
            )
            self.assertEqual(
                usage_venture2.value, 4234233423.0 / (1024 * 1024)
            )

    def test_fail_plugin(self):
        """ Testing not configured plugin """
        with mock.patch(
            'ralph_pricing.plugins.hamster.get_venture_capacity'
        ) as get_venture_capacity:
            get_venture_capacity.side_effect = mock_get_venture_capacity
            status, message, args = hamster_runner(
                today=datetime.datetime.today()
            )
            self.assertFalse(status)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import mock

from django.conf import settings
from django.test import TestCase
from ralph_pricing.models import DailyUsage, UsageType, Venture
from ralph_pricing.plugins.scaleme import (
    scaleme as scaleme_runner
)


def mock_get_ventures_capacities(date, url):
    return {
        "test_venture1": {
            "cache": 7878,
            "backend": 987987,
        },
        "test_venture2": {
            "cache": 666999,
            "backend": 12346699,
        },
    }


class TestScaleme(TestCase):
    def setUp(self):
        self.venture_1 = Venture(
            name='Test Venture1',
            symbol='test_venture1',
            venture_id='1',
        )
        self.venture_1.save()
        self.venture_2 = Venture(
            name='Test Venture2',
            symbol='test_venture2',
            venture_id='2',
        )
        self.venture_2.save()
        self.venture_3 = Venture(
            name='Test Venture3',
            symbol='test_venture3',
            venture_id='3',
        )
        self.venture_3.save()

    def test_set_usages(self):
        """Scaleme usages Test Case"""
        settings.SCALEME_API_URL = "/"
        with mock.patch(
            'ralph_pricing.plugins.scaleme.get_ventures_capacities'
        ) as get_ventures_capacities:
            get_ventures_capacities.side_effect = mock_get_ventures_capacities
            status, message, args = scaleme_runner(
                today=datetime.datetime.today()
            )
            self.assertTrue(status)
            usages = DailyUsage.objects.all()
            self.assertEqual(usages.count(), 4)

            usage_backend = UsageType.objects.get(
                name='Scaleme transforming an image 10000 events',
            )
            usage_cache = UsageType.objects.get(
                name='Scaleme serving image from cache 10000 events',
            )

            usages_venture1 = DailyUsage.objects.filter(
                pricing_venture=self.venture_1,
            )
            usages_venture2 = DailyUsage.objects.filter(
                pricing_venture=self.venture_2,
            )
            usages_venture3 = DailyUsage.objects.filter(
                pricing_venture=self.venture_3,
            )

            self.assertEqual(usages_venture1.count(), 2)
            usage_backend_venture1 = usages_venture1.filter(type=usage_backend)
            usage_cache_venture1 = usages_venture1.filter(type=usage_cache)
            self.assertEqual(usage_backend_venture1[0].value, 987987)
            self.assertEqual(usage_cache_venture1[0].value, 7878)

            self.assertEqual(usages_venture2.count(), 2)
            usage_backend_venture2 = usages_venture2.filter(type=usage_backend)
            usage_cache_venture2 = usages_venture2.filter(type=usage_cache)
            self.assertEqual(usage_backend_venture2[0].value, 12346699)
            self.assertEqual(usage_cache_venture2[0].value, 666999)

            self.assertEqual(usages_venture3.count(), 0)

    def test_fail_plugin(self):
        """Testing not configured plugin"""
        with mock.patch(
            'ralph_pricing.plugins.scaleme.get_ventures_capacities'
        ) as get_ventures_capacities:
            get_ventures_capacities.side_effect = mock_get_ventures_capacities
            status, message, args = scaleme_runner(
                today=datetime.datetime.today(),
            )
            self.assertFalse(status)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime

from ralph.util import plugin, api_pricing
from ralph_pricing.plugins import virtual, devices, cores


@plugin.register(chain='pricing', requires=['never run'])
def import_devices(**kwargs):
    """
        Imports devices, virtual usages and cores.
    """
    today = kwargs['today']
    end = datetime.date(2013, 3, 30)  # XXX Make it configurable!
    virtual_usages = virtual.get_usages()
    cores_usage = cores.get_usage()
    for data in api_pricing.devices_history(today, end):
        date = data['date']
        devices.update_device(data, date)
        if data['is_virtual']:
            virtual.update(data, virtual_usages, date)
        else:
            cores.update_cores(data, cores_usage, date)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.http import HttpResponseRedirect

from ralph_pricing.forms import ExtraCostFormSet
from ralph_pricing.menus import ventures_menu
from ralph_pricing.models import Venture
from ralph_pricing.views.base import Base


class ExtraCosts(Base):
    template_name = 'ralph_pricing/extra_costs.html'

    def __init__(self, *args, **kwargs):
        super(ExtraCosts, self).__init__(*args, **kwargs)
        self.formset = None
        self.venture = None
        self.venture_id = None

    def init_args(self):
        self.venture_id = self.kwargs.get('venture')
        if self.venture_id is not None:
            self.venture = get_object_or_404(Venture, id=self.venture_id)

    def post(self, *args, **kwargs):
        self.init_args()
        if self.venture:
            self.formset = ExtraCostFormSet(
                self.request.POST,
                queryset=self.venture.extracost_set.order_by('start', 'type'),
            )
            if self.formset.is_valid():
                for form in self.formset.extra_forms:
                    if form.has_changed():
                        form.instance.pricing_venture = self.venture
                self.formset.save()
                messages.success(self.request, "Extra costs updated.")
                return HttpResponseRedirect(self.request.path)
            else:
                messages.error(self.request, "Please fix the errors.")
        return super(ExtraCosts, self).get(*args, **kwargs)

    def get(self, *args, **kwargs):
        self.init_args()
        if self.venture:
            self.formset = ExtraCostFormSet(
                queryset=self.venture.extracost_set.order_by('start', 'type'),
            )
        return super(ExtraCosts, self).get(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ExtraCosts, self).get_context_data(**kwargs)
        context.update({
            'section': 'extra-costs',
            'sidebar_items': ventures_menu(
                '/pricing/extra-costs',
                self.venture_id
            ),
            'sidebar_selected': self.venture_id,
            'formset': self.formset,
        })
        return context



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import textwrap
import datetime
from optparse import make_option

from django.core.management.base import BaseCommand

from ralph.util import plugin


class Command(BaseCommand):
    """Retrieve data for pricing for today"""

    help = textwrap.dedent(__doc__).strip()
    requires_model_validation = True
    option_list = BaseCommand.option_list + (
        make_option(
            '--today',
            dest='today',
            default=None,
            help="Save the synchronised results under the specified date.",
        ),
        make_option(
            '--run-only',
            dest='run_only',
            default=None,
            help="Run only the selected plugin, ignore dependencies.",
        ),
    )

    def handle(self, today, run_only, *args, **options):
        from ralph_pricing import plugins  # noqa
        if today:
            today = datetime.datetime.strptime(today, '%Y-%m-%d').date()
        else:
            today = datetime.date.today()
        print('Synchronizing for {0}.'.format(today.isoformat()))
        if run_only:
            print('Running only {0}...'.format(run_only))
            success, message, context = plugin.run(
                'pricing',
                run_only,
                today=today,
            )
            print('{1}: {0}'.format(message, 'Done' if success else 'Failed'))
            return
        done = set()
        tried = set()
        while True:
            to_run = plugin.next('pricing', done) - tried
            if not to_run:
                break
            name = plugin.highest_priority('pricing', to_run)
            tried.add(name)
            print('Running {0}...'.format(name))
            success, message, context = plugin.run(
                'pricing', name, today=today,
            )
            print('{1}: {0}'.format(message, 'Done' if success else 'Failed'))
            if success:
                done.add(name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import datetime
import json
import re
import urllib
import urllib2


VENTURE = re.compile('venture:(?P<venture>.*);')


class Error(Exception):
    pass


class OpenStack(object):
    def __init__(self, url, user, password, region=''):
        self.auth_url = url
        self.user = user
        self.admin_url, self.public_url, self.auth_token = self.auth(
            password, region
        )

    def auth(self, password, region):
        auth_url = '/'.join([self.auth_url, 'v2.0/tokens'])
        auth_data = json.dumps({
            'auth': {
                'tenantName': self.user,
                'passwordCredentials': {
                    'username': self.user,
                    'password': password,
                },
            },
        })
        auth_headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
        }
        request = urllib2.Request(auth_url, auth_data, auth_headers)
        auth_replay = json.loads(urllib2.urlopen(request).read())
        for service in auth_replay['access']['serviceCatalog']:
            if service['name'] == 'nova':
                public_url = None
                for endpoint in service['endpoints']:
                    if not region or endpoint['region'] == region:
                        public_url = endpoint['publicURL']
                if not public_url:
                    raise Error('Service "nova" not available for this region')
            elif service['name'] == 'identity':
                for endpoint in service['endpoints']:
                    if not region or endpoint['region'] == region:
                        admin_url = endpoint['adminURL']
                if not admin_url:
                    raise Error(
                        'Service "keystone" not available for this region'
                    )
        auth_token = auth_replay['access']['token']['id']
        return admin_url, public_url, auth_token

    def query(self, query, url=None, **kwargs):
        query_args = urllib.urlencode(kwargs)
        query_headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'X-Auth-Project-Id': self.user,
            'X-Auth-Token': self.auth_token,
        }
        query_url = '/'.join([
            url or self.public_url,
            query,
        ]) + '?' + query_args
        request = urllib2.Request(query_url, headers=query_headers)
        return json.loads(urllib2.urlopen(request).read())

    def simple_tenant_usage(self, start=None, end=None):
        if end is None:
            end = datetime.datetime.now()
        if start is None:
            start = end - datetime.timedelta(hours=24)
        return self.query(
            'os-simple-tenant-usage',
            start=start.strftime('%Y-%m-%dT%H:%M:%S'),
            end=end.strftime('%Y-%m-%dT%H:%M:%S'),
        )['tenant_usages']

    def get_ventures(self):
        ventures = {}
        result = self.query('tenants', url=self.admin_url, limit=1000)
        for data in result['tenants']:
            if data['enabled']:
                venture = None
                description = data.get('description')
                if description:
                    result = re.match(VENTURE, description)
                    if result:
                        venture = result.group('venture')
                ventures[data['id']] = venture
        return ventures
